/* $Id: visor_getdata.cpp 99 2009-06-11 20:55:03Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "visor.h"

int get_summary_data(met_stat& s, time_t tbegin, time_t tend, met_station& e)
{
  size_t i, nobs, npar;
  time_t ti;
  met_report d;
  obs_data *o;

  // some control
  if (tbegin > tend)
    throw ("Fecha inicial > fecha final");

  // clean summary
  s.clean();

  if ((o = new obs_data[NUMOBS]) == NULL)
    {
      throw("No hay memoria suficiente para o[]");
    }

  /* The reading task is in blocks of NUMOBS data */
  ti = tbegin;
  nobs = 0;
  s.begin = tbegin;
  s.end = tend;

  while ((npar = get_db_obs_wide(e.db, &o[0], NUMOBS, ti, tend)) == NUMOBS)
    {
      ti = o[NUMOBS - 1].datime + o[NUMOBS - 1].interval * 60; //assumed next obs interval is the same
      for (i = 0; i < npar; i++)
        {
          // first obs may be prior to tbegin, we then
          // substitute it by an interpolated real_data
          // Same for last obs in the wide interval
          if (o[i].datime < tbegin)
            {
              // It is usually first obs
              if (interp_met_report(d, &o[i], 2, tbegin, e))
                summarize_met_report(s, d);
            }
          else if (o[i].datime > tend)
            {
              // it is last obs
              if( interp_met_report(d, &o[i - 1], 2, tend, e))
                summarize_met_report(s, d);
            }
          else
            {
              d = met_report(&o[i], e);
              summarize_met_report(s, d);
            }
        }
    }

  // last 4096 block

  for (i = 0; i < npar; i++)
    {
      // first obs may be prior to tbegin, we then
      // substitute it by an interpolated real_data
      // Same for last obs in the wide interval
      if (o[i].datime < tbegin)
        {
          // It is usually first obs
          if (interp_met_report(d, &o[i], 2, tbegin, e))
            summarize_met_report(s, d);
        }
      else if (o[i].datime > tend)
        {
          // it is last obs
          if (interp_met_report(d, &o[i - 1], 2, tend, e))
            summarize_met_report(s, d);
        }
      else
        {
          d = met_report( &o[i], e);
          summarize_met_report(s, d);
        }
    }

  delete [] o;

  return 1;
}

// Esta version ya tiene la memoria de datos alojada, con dimension numobs
// devuelve el numero de datos leidos
int get_summary_data(met_report *d, size_t numobs, met_stat& s, time_t tbegin, time_t tend, met_station& e)
{
  size_t i, j, npar;
  time_t ti;
  obs_data *o;

  // some control
  if (tbegin > tend)
    throw ("Fecha inicial > fecha final");

  // clean summary
  s.clean();

  if ((o = new obs_data[NUMOBS]) == NULL)
    {
      throw("No hay memoria suficiente para o[]");
    }

  /* The reading task is in blocks of NUMOBS data */
  ti = tbegin;
  s.begin = tbegin;
  s.end = tend;
  j = 0;

  while ((npar = get_db_obs_wide(e.db, &o[0], NUMOBS, ti, tend)) == NUMOBS)
    {
      ti = o[NUMOBS - 1].datime + o[NUMOBS - 1].interval * 60; //assumed next obs interval is the same
      for (i = 0; (i < npar) && (j < numobs) ; i++)
        {
          // first obs may be prior to tbegin, we then
          // substitute it by an interpolated real_data
          // Same for last obs in the wide interval
          if (o[i].datime < tbegin)
            {
              // It is usually first obs
              if (interp_met_report(d[j], &o[i], 2, tbegin, e))
                summarize_met_report(s, d[j++]);
            }
          else if (o[i].datime > tend)
            {
              // it is last obs
              if( interp_met_report(d[j], &o[i - 1], 2, tend, e))
                summarize_met_report(s, d[j++]);
            }
          else
            {
              d[j] = met_report(&o[i], e);
              summarize_met_report(s, d[j++]);
            }
        }
    }

  // last 4096 block

  for (i = 0; (i < npar) && (j < numobs); i++)
    {
      // first obs may be prior to tbegin, we then
      // substitute it by an interpolated real_data
      // Same for last obs in the wide interval
      if (o[i].datime < tbegin)
        {
          // It is usually first obs
          if (interp_met_report(d[j], &o[i], 2, tbegin, e))
            summarize_met_report(s, d[j++]);
        }
      else if (o[i].datime > tend)
        {
          // it is last obs
          if (interp_met_report(d[j], &o[i - 1], 2, tend, e))
            summarize_met_report(s, d[j++]);
        }
      else
        {
          d[j] = met_report( &o[i], e);
          summarize_met_report(s, d[j++]);
        }
    }

  delete [] o;

  return j;
}

// esta rutina devuelve el número de datos alojados (y leidos) No hace summary
int get_data(met_report **d, time_t tbegin, time_t tend, met_station& e)
{
  size_t i, j, npar, ndat;
  time_t ti;
  obs_data *o;

  // some control
  if (tbegin > tend)
    throw ("Fecha inicial > fecha final");

  if ((o = new obs_data[NUMOBS]) == NULL)
    {
      throw("No hay memoria suficiente para o[]");
    }

  // Cuantos datos tenemos en el intervalo pedido
  if ((ndat = count_db_obs_wide(e.db, tbegin, tend)) == 0)
    {
      return 0;
    }

  if ( *d != NULL)
    {
      throw("El puntero d no está vacio para alojar memoria");
    }
 
  if ((*d = new met_report[ndat]) == NULL)
    {
      throw("No hay memoria suficiente para d[]");
    }
  

  /* The reading task is in blocks of NUMOBS data */
  ti = tbegin;
  j = 0;

  while ((npar = get_db_obs_wide(e.db, &o[0], NUMOBS, ti, tend)) == NUMOBS)
    {
      ti = o[NUMOBS - 1].datime + o[NUMOBS - 1].interval * 60; //assumed next obs interval is the same
      for (i = 0; (i < npar) && (j < ndat) ; i++)
        {
          // first obs may be prior to tbegin, we then
          // substitute it by an interpolated real_data
          // Same for last obs in the wide interval
          if (o[i].datime < tbegin)
            {
              // It is usually first obs
              if (interp_met_report((*d)[j], &o[i], 2, tbegin, e))
                j++;
            }
          else if (o[i].datime > tend)
            {
              // it is last obs
              if( interp_met_report((*d)[j], &o[i - 1], 2, tend, e))
                j++;
            }
          else
            {
              (*d)[j] = met_report(&o[i], e);
              j++;
            }
        }
    }

  // last 4096 block

  for (i = 0; (i < npar) && (j < ndat); i++)
    {
      // first obs may be prior to tbegin, we then
      // substitute it by an interpolated real_data
      // Same for last obs in the wide interval
      if (o[i].datime < tbegin)
        {
          // It is usually first obs
          if (interp_met_report((*d)[j], &o[i], 2, tbegin, e))
            j++;
        }
      else if (o[i].datime > tend)
        {
          // it is last obs
          if (interp_met_report((*d)[j], &o[i - 1], 2, tend, e))
            j++;
        }
      else
        {
          (*d)[j] = met_report( &o[i], e);
          j++;
        }
    }

  delete [] o;

  return j;
}
