/* $Id: visor_io.cpp 126 2009-06-14 18:08:09Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*!
  \file visor_io.cpp
  \brief file with the routines that read GET/POST arguments and set parameters accordingly
*/
#include "visor.h"

char GARGS[NGARGS][8]={"gtmi","gtmo","ghui","ghuo","gwdi","gwsp","gprs","gspr","gprc"};
int GARGS_M[NGARGS] = {MASK_GRA_TEMP_INT, MASK_GRA_TEMP_EXT, MASK_GRA_HUM_INT, MASK_GRA_HUM_EXT,
                       MASK_GRA_WDIR, MASK_GRA_WSPD, MASK_GRA_PRES, MASK_GRA_SEA_PRES, MASK_GRA_RAIN
                      };
char GARGS_NAME[NGARGS][24]={"T. interior.","T. exterior","H. interior","H. exterior.","Dir. viento","Vel. viento","Presión.","Pres. n. mar","Precipitación"};

int read_arg(string& arg, Cgicc& cgi, const char *name)
{
  form_iterator argi;

  if ( (argi = cgi.getElement(name)) != cgi.getElements().end())
    {
      arg = **argi;
      return 1;
    }
  return 0;
}

int read_getargs(Cgicc& cgi, met_station& s)
{
  int i;
  long int la;
  char *c;
  time_t ti;
  struct tm tim;
  string strx;

  char fecha[MAX_ARGLEN];

  // Default or initial values
  SHOWLAST = 0;
  NOW = 1;
  TBEGIN = 0;
  TEND = 0;
  GTASK_MASK = 0;

  if (read_arg(Fecha, cgi, "fecha") == 0)
    {
      ti = time(NULL);
      memset(&tim, 0, sizeof(struct tm));
      gmtime_r(&ti, &tim);
      strftime(fecha, MAX_ARGLEN, "%Y%m%d", &tim);
      Fecha = fecha;
      SHOWLAST = 1;
      strcat(fecha,"0000");
      memset(&tim, 0, sizeof(struct tm));
      strptime(fecha,"%Y%m%d%H%M", &tim);
      TBEGIN = timegm(&tim);
      TEND = TBEGIN + 86340;
      //printf("%s\n", Fecha.c_str());
    }
  else
    {
      if (strlen(Fecha.c_str()) != 8)
        {
          ti = time(NULL);
          memset(&tim, 0, sizeof(struct tm));
          gmtime_r(&ti, &tim);
          strftime(fecha, MAX_ARGLEN, "%Y%m%d", &tim);
          Fecha = fecha;
          SHOWLAST = 1;
          strcat(fecha,"0000");
          memset(&tim, 0, sizeof(struct tm));
          strptime(fecha,"%Y%m%d%H%M", &tim);
          TBEGIN = timegm(&tim);
          TEND = TBEGIN + 86340;
          //printf("%s\n", Fecha.c_str());
        }
      else
        {
          strcpy(fecha,Fecha.c_str());
          memset(&tim, 0, sizeof (struct tm));
          strcat(fecha,"0000");
          strptime(fecha,"%Y%m%d%H%M", &tim);
          TBEGIN = timegm(&tim);
          TEND = TBEGIN + 86340;
        }
    }

  if (read_arg(strx, cgi, "begin"))
    {
      strcpy(fecha, strx.c_str());

      if (strlen(fecha) != 12 || strspn(optarg, "0123456789") != 12)
        {
          throw "Begin date must be in YYYYMMDDHHmm format<br>\n";
        }
      memset(&tim, 0, sizeof(struct tm));
      strptime(fecha, "%Y%m%d%H%M%S", &tim);
      TBEGIN = timegm(&tim);
    }

  if (read_arg(strx, cgi, "end"))
    {
      strcpy(fecha, strx.c_str());

      if (strlen(fecha) != 12 || strspn(optarg, "0123456789") != 12)
        {
          throw "End date must be in YYYYMMDDHHmm format<br>\n";
        }
      memset(&tim, 0, sizeof(struct tm));
      strptime(fecha, "%Y%m%d%H%M%S", &tim);
      TEND = timegm(&tim);
    }

  if (read_arg(strx, cgi, "nd"))
    {
      strcpy(fecha, strx.c_str());
      errno = 0;
      la = strtol( fecha, &c, 10);
      if (errno)
        ND = 7;
      else
        ND = la;
    }
  else
    ND = 7;

  if (read_arg(strx, cgi, "ndg"))
    {
      strcpy(fecha, strx.c_str());
      errno = 0;
      la = strtol( fecha, &c, 10);
      if (errno)
        NDG = 2;
      else
        NDG = la;
    }
  else
    NDG = 2;

  if (read_arg(strx, cgi, "last"))
    {
      if (strcmp("y",strx.c_str()) == 0)
        NOW = 1;
      else if (strcmp("n",strx.c_str()) == 0)
        NOW = 0;
    }

  if (read_arg(s.nocal, cgi, "Nocal") == 0)
    {
      s.nocal = "no";
    }

  if (read_arg(s.confile, cgi, "Confile") == 0)
    {
      s.confile = "/etc/wwsr_freewestat.conf";
    }

  // Graphics to plot
  for (i = 0; i < NGARGS; i++)
    {
      if (read_arg(strx, cgi, GARGS[i]) && (strcmp("y",strx.c_str()) == 0))
        {
          GTASK_MASK |= GARGS_M[i];
        }
    }

  // If no arguments, assign default Graphic task
  if (GTASK_MASK == 0)
    GTASK_MASK = (MASK_GRA_TEMP_EXT | MASK_GRA_HUM_EXT | MASK_GRA_WDIR | MASK_GRA_WSPD |
                  MASK_GRA_PRES | MASK_GRA_RAIN);


  return 1;
}
