/* $Id: visor_main.cpp 133 2009-07-02 21:33:07Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*!
  \file visor_main.cpp
  \brief main file for visor cgi binary
*/
#include "visor.h"

string Fecha;

int  SHOWLAST, ND, NDG, GTASK_MASK, NOW;
char REMOTEHOST[64];
time_t TBEGIN, TEND;

int main(/*int argc, char **argv*/)
{
  int res;
  Cgicc cgi;
  met_station E(MMASK_WWSR);
  met_stat S;
  metgra G;
  string fecha, nocal;
  struct obs_data *Obs;
  met_report d0;

  try
    {
      // Send HTTP header
      cout << HTTPHTMLHeader() << endl;

      // Set up the HTML document
      cout << html() << head() << title("Visor") << endl;
      cout << meta().set("http-equiv","Content-Type").set("content","text/html ; charset=UTF-8") << endl;
      cout << meta().set("http-equiv","pragma").set("content","NO-CACHE") << endl;
      print_css();

      // Get a pointer to the environment
      const CgiEnvironment& env = cgi.getEnvironment();
      string remotehost = env.getRemoteAddr();
      strcpy(REMOTEHOST, remotehost.c_str());

      // read args
      if (read_getargs(cgi, E) == 0)
        {
          throw "No se ha podido leer los argumentos";
        }

      // get station conf
      if (E.get_station_conf() == 0)
        {
          throw "No se ha podido leer la configuración de la estación";
        }

      if (E.get_station_cal())
        {
          throw "No se ha podido leer el calibrado";
        }

      // open database
      if ((res = E.open_met_db()))
        {
          throw "No se ha podido abrir la base de datos";
        }

      // reserva memoria
      Obs = new struct obs_data[2];

      if (NOW)
        {
          cout << meta().set("http-equiv","refresh").set("content","600") << endl;
        }
      cout << head() << body() << endl;

      // prologo
      print_prolog(E);

      // Print brief info for station
      print_info_station(E);

      if (NOW)
        {
          // Muestra la ultima observación
          if (get_db_obs_last(E.db, &Obs[0], 1))
            {
              d0 = met_report(&Obs[0], E);
              show_met_report_txt(d0, E);
            }
        }

      // Calcula e imprime el resultado del dia en cuestion
      //if (get_summary_data(d, NUMOBS, S, TBEGIN, TEND, E))
      if (ND && get_summary_data(S, TBEGIN, TEND, E))
        {
          print_met_stat (S, MMASK_WWSR);
        }

      if (NDG)
        {
          if (SHOWLAST)
            data_plot(G, E, S, time(NULL), NDG, GTASK_MASK);
          else
            data_plot(G, E, S, TBEGIN + 86400, NDG, GTASK_MASK);
        }

      // Imprime una tabla con resúmenes diarios de los ND dias anteriores
      if (ND > 0 && ND < 367)
        print_tableres(E, TBEGIN, ND, MMASK_WWSR);

      // Clear memory
      delete [] Obs;

      // close database
      close_db(E.db, NULL);


      // Close the HTML document

      print_epilog();
      cout << body() << html();

    }
  catch (exception& e)
    {
      // handle any errors - omitted for brevity
      cout << e.what() << endl;
      cout << "</BODY></HTML>" << endl;
    }
  catch (char const *e)
    {
      // handle any errors - omitted for brevity
      cout << e << endl;
      cout << "</BODY></HTML>" << endl;
    }

}


