/* $Id: visor_metgra.cpp 121 2009-06-14 16:19:21Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "visor.h"

/*!
  \file visor.h
  \brief include data to deal with classes and structures in metgra lib
*/

// pasa de met_report a metgra_report
void met2metgra(metgra_report& g, met_report& r)
{
  g.mask = r.mask;
  g.datime = r.datime;
  g.interval = r.interval;
  g.hum_int = r.hum_int;
  g.temp_int = r.temp_int;
  g.tdew_int = r.tdew_int;
  g.hum_out = r.hum_out;
  g.temp_out = r.temp_out;
  g.tdew_out = r.tdew_out;
  g.press = r.press;
  g.sea_press = r.sea_press;
  g.wspd = r.wspd;
  g.wgust = r.wgust;
  g.wdir = r.wdir;
  g.wdgust = r.wdir;
  g.rain = r.rain;
}


// prepara algunos miembros comunes en todas las gráficas
int prepara_metgra(metgra& g, met_station& e, met_stat &s, time_t tbegin, int ndays)
{
  size_t nd;
  char *c;

  // secppix secpvl
  switch (ndays)
    {
    case 1:
      g.secppix = 120;
      g.secpvl = 3600;
      break;
    case 2:
      g.secppix = 240;
      g.secpvl = 7200;
      break;
    case 3:
      g.secppix = 360;
      g.secpvl = 10800;
      break;
    case 7:
      g.secppix = 600;
      g.secpvl = 21600;
      break;
    default:
      g.secppix = 240;
      g.secpvl = 7200;
      break;
    }

  // tiempos
  g.tbegin = tbegin - g.secpvl;
  g.tend = tbegin + 86400 * ndays;

  // tamaños comunes
  g.mx = 30;
  g.dsx = (g.tend - g.tbegin) / g.secppix;
  g.sx = g.dsx + 2 * g.mx;
  g.myb = 30;
  g.myt = 2 * g.myb;

  // datos
  {
    time_t tend;
    met_report *dd = NULL;

    if ((nd = get_data(&dd, g.tbegin, g.tend, e)) == 0)
      {
        return 0;
      }

    if ((g.dat = new metgra_report[nd]) == NULL)
      {
        delete [] dd;
        throw ("No hay espacio para alojar g.dat");
      }

    // clean stat
    s.clean();
    s.begin = g.tbegin;
    s.end = g.tend;

    for (size_t i = 0; i < nd; i++)
      {
        met2metgra(g.dat[i], dd[i]);
        summarize_met_report(s, dd[i]);
      }
    g.ndat = nd;
    g.t0 = g.dat[0].datime;
    delete [] dd;
  }

  // orto y ocaso
  g.set_astr(g.tbegin, e.conf.latitude, e.conf.longitude);

  // Título
  strcpy(g.title, e.conf.name);

  // font
  if ((c = get_fontpath()) == NULL)
    {
      throw ("No se encuentra una fuente TTF apropiada");
    }
  else
    strcpy(g.font, c);

  return 1;
}

int plot_temp_ext(metgra& g, met_stat &s)
{
  time_t hmx, hmn;
  double tx, tn;
  char *c;

  // mascara
  g.mask = MASK_GRA_TEMP_EXT;

  // valores extremos en grafica
  tx = s.temp_out.vmax;
  hmx = s.temp_out.tvmax;
  tn = s.tdew_out.vmin;
  hmn = s.tdew_out.tvmax;

  g.et = ceil((tx + 1.0) / 5.0) * 5.0;
  g.eb = floor((tn - 1.0)/ 5.0) * 5.0;

  // prepara tamaños
  g.datppix = 0.1 ; // grados por pixel vertical

  g.dsy = (g.et - g.eb) / g.datppix + 0.5;

  g.sy = g.dsy + g.myt + g.myb;

  // Prepara y fichero
  strcpy(g.subtit, "Temperatura y punto de rocío exterior");

  // Nombre del fichero grafico
  c = get_dirtmp();

  if (c == NULL)
    sprintf(g.file,"/tmp/temp_out.%s.png",REMOTEHOST);
  else
    sprintf(g.file,"%s/temp_out.%s.png", c, REMOTEHOST);

  // una vez que está todo preparado se pinta la grafica
  g.plot_graf();


  return 1;
}

int plot_temp_int(metgra& g, met_stat &s)
{
  time_t hmx, hmn;
  double tx, tn;
  char *c;

  // mascara
  g.mask = MASK_GRA_TEMP_INT;

  // valores extremos en grafica
  tx = s.temp_int.vmax;
  hmx = s.temp_int.tvmax;
  tn = s.tdew_int.vmin;
  hmn = s.tdew_int.tvmax;

  g.et = ceil((tx + 1.0) / 5.0) * 5.0;
  g.eb = floor((tn - 1.0)/ 5.0) * 5.0;

  // prepara tamaños
  g.datppix = 0.1 ; // grados por pixel vertical

  g.dsy = (g.et - g.eb) / g.datppix + 0.5;

  g.sy = g.dsy + g.myt + g.myb;

  // Prepara y fichero
  strcpy(g.subtit, "Temperatura y punto de rocío interior");

  // Nombre del fichero grafico
  c = get_dirtmp();

  if (c == NULL)
    sprintf(g.file,"/tmp/temp_int.%s.png", REMOTEHOST);
  else
    sprintf(g.file,"%s/temp_int.%s.png", c, REMOTEHOST);

  // una vez que está todo preparado se pinta la grafica
  g.plot_graf();


  return 1;
}

int plot_hum_ext(metgra& g, met_stat &s)
{
  time_t hmx, hmn;
  double tx, tn;
  char *c;

  // mascara
  g.mask = MASK_GRA_HUM_EXT;

  // valores extremos en grafica
  tx = s.hum_out.vmax;
  hmx = s.hum_out.tvmax;
  tn = s.hum_out.vmin;
  hmn = s.hum_out.tvmax;

  g.et = ceil((tx + 1.0) / 10.0) * 10.0;
  g.eb = floor((tn - 1.0)/ 10.0) * 10.0;

  if (g.et > 100.0)
    g.et = 100.0;
  if (g.eb < 0.0)
    g.eb = 0.0;


  // prepara tamaños
  g.datppix = 0.3 ; // grados por pixel vertical

  g.dsy = (g.et - g.eb) / g.datppix + 0.5;

  g.sy = g.dsy + g.myt + g.myb;

  // Prepara y fichero
  strcpy(g.subtit, "Humedad relativa exterior");

  // Nombre del fichero grafico
  c = get_dirtmp();

  if (c == NULL)
    sprintf(g.file,"/tmp/hum_out.%s.png", REMOTEHOST);
  else
    sprintf(g.file,"%s/hum_out.%s.png", c, REMOTEHOST);

  // una vez que está todo preparado se pinta la grafica
  g.plot_graf();


  return 1;
}

int plot_hum_int(metgra& g, met_stat &s)
{
  time_t hmx, hmn;
  double tx, tn;
  char *c;

  // mascara
  g.mask = MASK_GRA_HUM_INT;

  // valores extremos en grafica
  tx = s.hum_int.vmax;
  hmx = s.hum_int.tvmax;
  tn = s.hum_int.vmin;
  hmn = s.hum_int.tvmax;

  g.et = ceil((tx + 1.0) / 10.0) * 10.0;
  g.eb = floor((tn - 1.0)/ 10.0) * 10.0;

  if (g.et > 100.0)
    g.et = 100.0;
  if (g.eb < 0.0)
    g.eb = 0.0;


  // prepara tamaños
  g.datppix = 0.3 ; // grados por pixel vertical

  g.dsy = (g.et - g.eb) / g.datppix + 0.5;

  g.sy = g.dsy + g.myt + g.myb;

  // Prepara y fichero
  strcpy(g.subtit, "Humedad relativa interior");

  // Nombre del fichero grafico
  c = get_dirtmp();

  if (c == NULL)
    sprintf(g.file,"/tmp/hum_int.%s.png", REMOTEHOST);
  else
    sprintf(g.file,"%s/hum_int.%s.png", c, REMOTEHOST);

  // una vez que está todo preparado se pinta la grafica
  g.plot_graf();


  return 1;
}

int plot_press(metgra& g, met_stat &s)
{
  time_t hmx, hmn;
  double tx, tn;
  char *c;

  // mascara
  g.mask = MASK_GRA_PRES;

  // valores extremos en grafica
  tx = s.press.vmax;
  hmx = s.press.tvmax;
  tn = s.press.vmin;
  hmn = s.press.tvmax;

  g.et = ceil((tx + 1.0) / 5.0) * 5.0;
  g.eb = floor((tn - 1.0)/ 5.0) * 5.0;

  // prepara tamaños
  g.datppix = 0.1 ; // Hpa por pixel vertical

  g.dsy = (g.et - g.eb) / g.datppix + 0.5;

  g.sy = g.dsy + g.myt + g.myb;

  // Prepara y fichero
  strcpy(g.subtit, "Presión atmosférica al nivel de la estación");

  // Nombre del fichero grafico
  c = get_dirtmp();

  if (c == NULL)
    sprintf(g.file,"/tmp/press.%s.png", REMOTEHOST);
  else
    sprintf(g.file,"%s/press.%s.png", c, REMOTEHOST);

  // una vez que está todo preparado se pinta la grafica
  g.plot_graf();

  return 1;
}

int plot_prec(metgra& g, met_stat &s)
{
  char *c;

  // mascara
  g.mask = MASK_GRA_RAIN;

  // valores extremos en grafica
  g.et = 10.0;
  g.eb = 0.0;

  // prepara tamaños
  g.datppix = 0.025; // Hpa por pixel vertical

  g.dsy = (g.et - g.eb) / g.datppix + 0.5;

  g.sy = g.dsy + g.myt + g.myb;

  // Prepara y fichero
  strcpy(g.subtit, "Precipitación acumulada");

  // Nombre del fichero grafico
  c = get_dirtmp();

  if (c == NULL)
    sprintf(g.file,"/tmp/prec.%s.png",REMOTEHOST);
  else
    sprintf(g.file,"%s/prec.%s.png", c, REMOTEHOST);

  // una vez que está todo preparado se pinta la grafica
  g.plot_graf();

  return 1;
}


int plot_sea_press(metgra& g, met_stat &s)
{
  time_t hmx, hmn;
  double tx, tn;
  char *c;

  // mascara
  g.mask = MASK_GRA_SEA_PRES;

  // valores extremos en grafica
  tx = s.sea_press.vmax;
  hmx = s.sea_press.tvmax;
  tn = s.sea_press.vmin;
  hmn = s.sea_press.tvmax;

  g.et = ceil((tx + 1.0) / 5.0) * 5.0;
  g.eb = floor((tn - 1.0)/ 5.0) * 5.0;

  // prepara tamaños
  g.datppix = 0.1 ; // Hpa por pixel vertical

  g.dsy = (g.et - g.eb) / g.datppix + 0.5;

  g.sy = g.dsy + g.myt + g.myb;

  // Prepara y fichero
  strcpy(g.subtit, "Presión atmosférica reducida al nivel del mar");

  // Nombre del fichero grafico
  c = get_dirtmp();

  if (c == NULL)
    sprintf(g.file,"/tmp/sea_press.%s.png", REMOTEHOST);
  else
    sprintf(g.file,"%s/sea_press.%s.png", c, REMOTEHOST);

  // una vez que está todo preparado se pinta la grafica
  g.plot_graf();

  return 1;
}


int plot_wspd(metgra& g, met_stat &s)
{
  double tx, tn;
  char *c;

  // mascara
  g.mask = MASK_GRA_WSPD;

  // valores extremos en grafica
  tx = s.wgust.vmax;
  tn = 0.0;

  g.et = ceil((tx + 1.0) / 10.0) * 10.0;
  g.eb = 0.0;

  // prepara tamaños
  g.datppix = 0.2 ; // grados por pixel vertical

  g.dsy = (g.et - g.eb) / g.datppix + 0.5;

  g.sy = g.dsy + g.myt + g.myb;

  // Prepara y fichero
  strcpy(g.subtit, "Velocidad del viento y racha máxima");

  // Nombre del fichero grafico
  c = get_dirtmp();

  if (c == NULL)
    sprintf(g.file,"/tmp/wspd.%s.png", REMOTEHOST);
  else
    sprintf(g.file,"%s/wspd.%s.png", c, REMOTEHOST);

  // una vez que está todo preparado se pinta la grafica
  g.plot_graf();


  return 1;
}


int plot_dirw(metgra& g, met_stat &s)
{
  char *c;

  // mascara
  g.mask = MASK_GRA_WDIR;

  g.et = 360.0;
  g.eb = 0.0;

  // prepara tamaños
  g.datppix = 2.5 ; // grados por pixel vertical

  g.dsy = (g.et - g.eb) / g.datppix + 0.5;

  g.sy = g.dsy + g.myt + g.myb;

  // Prepara y fichero
  strcpy(g.subtit, "Dirección del viento");

  // Nombre del fichero grafico
  c = get_dirtmp();

  if (c == NULL)
    sprintf(g.file,"/tmp/wdir.%s.png", REMOTEHOST);
  else
    sprintf(g.file,"%s/wdir.%s.png", c, REMOTEHOST);

  // una vez que está todo preparado se pinta la grafica
  g.plot_graf();


  return 1;
}


int data_plot(metgra& g, met_station& e, met_stat &s, time_t tref, int ndays, int mask)
{
  time_t taux;
  size_t secpvl;

  switch (ndays)
    {
    case 1:
      secpvl = 3600;
      break;
    case 2:
      secpvl = 7200;
      break;
    case 3:
      secpvl = 10800;
      break;
    case 7:
      secpvl = 21600;
      break;
    default:
      return 0;
    }

  taux = (((tref + 1800) / secpvl) * secpvl) + secpvl - 86400 * ndays;

  prepara_metgra(g, e, s, taux, ndays); // prepara datos comunes

  cout << "<br><TABLE bgcolor='#CCDDFF' border='0' align='center'>" << endl;

  if (mask & MASK_GRA_TEMP_EXT)
    {
      // grafico de temperatura externa
      plot_temp_ext(g, s); 

      cout << "<TR><TD>" << endl;
      cout << "<img src=\"/tmp/temp_out."<< REMOTEHOST << ".png\">" << endl;
      cout << "</TD></TR>" << endl;
    }

  if (mask & MASK_GRA_TEMP_INT)
    {
      // grafico de temperatura interna
      plot_temp_int(g, s); 

      cout << "<TR><TD>" << endl;
      cout << "<img src=\"/tmp/temp_int."<< REMOTEHOST << ".png\">" << endl;
      cout << "</TD></TR>" << endl;
    }

  if (mask & MASK_GRA_HUM_EXT)
    {
      // grafico de temperatura interna
      plot_hum_ext(g, s); 

      cout << "<TR><TD>" << endl;
      cout << "<img src=\"/tmp/hum_out."<< REMOTEHOST << ".png\">" << endl;
      cout << "</TD></TR>" << endl;
    }

  if (mask & MASK_GRA_HUM_INT)
    {
      // grafico de temperatura interna
      plot_hum_int(g, s); 

      cout << "<TR><TD>" << endl;
      cout << "<img src=\"/tmp/hum_int."<< REMOTEHOST << ".png\">" << endl;
      cout << "</TD></TR>" << endl;
    }

  if (mask & MASK_GRA_WDIR)
    {
      // grafico de temperatura interna
      plot_dirw(g, s); 

      cout << "<TR><TD>" << endl;
      cout << "<img src=\"/tmp/wdir."<< REMOTEHOST << ".png\">" << endl;
      cout << "</TD></TR>" << endl;
    }

  if (mask & MASK_GRA_WSPD)
    {
      // grafico de temperatura interna
      plot_wspd(g, s); 

      cout << "<TR><TD>" << endl;
      cout << "<img src=\"/tmp/wspd."<< REMOTEHOST << ".png\">" << endl;
      cout << "</TD></TR>" << endl;
    }

  if (mask & MASK_GRA_RAIN)
    {
      // grafico de temperatura interna
      plot_prec(g, s); 

      cout << "<TR><TD>" << endl;
      cout << "<img src=\"/tmp/prec."<< REMOTEHOST << ".png\">" << endl;
      cout << "</TD></TR>" << endl;
    }

  if (mask & MASK_GRA_PRES)
    {
      // grafico de temperatura interna
      plot_press(g, s); 

      cout << "<TR><TD>" << endl;
      cout << "<img src=\"/tmp/press."<< REMOTEHOST << ".png\">" << endl;
      cout << "</TD></TR>" << endl;
    }

  if (mask & MASK_GRA_SEA_PRES)
    {
      // grafico de temperatura interna
      plot_sea_press(g, s); 

      cout << "<TR><TD>" << endl;
      cout << "<img src=\"/tmp/sea_press."<< REMOTEHOST << ".png\">" << endl;
      cout << "</TD></TR>" << endl;
    }


  cout << "</TABLE>" << endl;

  return 1;
}
