/* $Id: visor_stat.cpp 98 2009-06-10 05:08:31Z gbv $ */ 
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "visor.h"

int summarize_met_report(met_stat& s, const met_report& r)
{
  double aux, intervalo;

  // Date and time must be in the intervalo
  if ((r.mask & MMASK_DATIME) == 0)
    return 0;

  if ((r.datime > s.end) || (r.datime < s.begin))
    return 0;

  if (s.n == 0)
    intervalo= 0.0;
  else
    intervalo= (double)r.datime - (double) s.latest;

  // hum_int
  if (r.mask & MMASK_HUM_INT)
    {
      s.hum_int.tiempo += intervalo;
      s.hum_int.acum += r.hum_int * intervalo;
      if (s.hum_int.n == 0 || s.hum_int.vmax < r.hum_int)
        {
          s.hum_int.vmax = r.hum_int;
          s.hum_int.tvmax = r.datime;
        }
      if (s.hum_int.n == 0 || s.hum_int.vmin > r.hum_int)
        {
          s.hum_int.vmin = r.hum_int;
          s.hum_int.tvmin = r.datime;
        }
      (s.hum_int.n)++;
    }

  // temp_int
  if (r.mask & MMASK_TEMP_INT)
    {
      s.temp_int.tiempo += intervalo;
      s.temp_int.acum += (r.temp_int * intervalo);
      if (s.temp_int.n == 0 || s.temp_int.vmax < r.temp_int)
        {
          s.temp_int.vmax = r.temp_int;
          s.temp_int.tvmax = r.datime;
        }
      if (s.temp_int.n == 0 || s.temp_int.vmin > r.temp_int)
        {
          s.temp_int.vmin = r.temp_int;
          s.temp_int.tvmin = r.datime;
        }
      (s.temp_int.n)++;
    }

  // tdew_int
  if ((r.mask & MMASK_TEMP_INT) && (r.mask & MMASK_HUM_INT))
    {
      s.tdew_int.tiempo += intervalo;
      s.tdew_int.acum += (r.tdew_int * intervalo);
      if (s.tdew_int.n == 0 || s.tdew_int.vmax < r.tdew_int)
        {
          s.tdew_int.vmax = r.tdew_int;
          s.tdew_int.tvmax = r.datime;
        }
      if (s.tdew_int.n == 0 || s.tdew_int.vmin > r.tdew_int)
        {
          s.tdew_int.vmin = r.tdew_int;
          s.tdew_int.tvmin = r.datime;
        }
      (s.tdew_int.n)++;
    }

  // hum_out
  if (r.mask & MMASK_HUM_EXT)
    {
      s.hum_out.tiempo += intervalo;
      s.hum_out.acum += (r.hum_out * intervalo);
      if (s.hum_out.n == 0 || s.hum_out.vmax < r.hum_out)
        {
          s.hum_out.vmax = r.hum_out;
          s.hum_out.tvmax = r.datime;
        }
      if (s.hum_out.n == 0 || s.hum_out.vmin > r.hum_out)
        {
          s.hum_out.vmin = r.hum_out;
          s.hum_out.tvmin = r.datime;
        }
      (s.hum_out.n)++;
    }

  // temp_out
  if (r.mask & MMASK_TEMP_EXT)
    {
      s.temp_out.tiempo += intervalo;
      s.temp_out.acum += (r.temp_out * intervalo);
      //printf("%lf %lf %lf<br>\n", intervalo, s.temp_out.tiempo, s.temp_out.acum);
      if (s.temp_out.n == 0 || s.temp_out.vmax < r.temp_out)
        {
          s.temp_out.vmax = r.temp_out;
          s.temp_out.tvmax = r.datime;
        }
      if (s.temp_out.n == 0 || s.temp_out.vmin > r.temp_out)
        {
          s.temp_out.vmin = r.temp_out;
          s.temp_out.tvmin = r.datime;
        }
      (s.temp_out.n)++;
    }

  // tdew_out
  if ((r.mask & MMASK_TEMP_EXT) && (r.mask & MMASK_HUM_EXT))
    {
      s.tdew_out.tiempo += intervalo;
      s.tdew_out.acum += (r.tdew_out * intervalo);
      if (s.tdew_out.n == 0 || s.tdew_out.vmax < r.tdew_out)
        {
          s.tdew_out.vmax = r.tdew_out;
          s.tdew_out.tvmax = r.datime;
        }
      if (s.tdew_out.n == 0 || s.tdew_out.vmin > r.tdew_out)
        {
          s.tdew_out.vmin = r.tdew_out;
          s.tdew_out.tvmin = r.datime;
        }
      (s.tdew_out.n)++;
    }

  // pressure at station level 
  if (r.mask & MMASK_PRES)
    {
      s.sea_press.tiempo += intervalo;
      s.sea_press.acum += (r.sea_press * intervalo);
      if (s.sea_press.n == 0 || s.sea_press.vmax < r.sea_press)
        {
          s.sea_press.vmax = r.sea_press;
          s.sea_press.tvmax = r.datime;
        }
      if (s.sea_press.n == 0 || s.sea_press.vmin > r.sea_press)
        {
          s.sea_press.vmin = r.sea_press;
          s.sea_press.tvmin = r.datime;
        }
      (s.sea_press.n)++;
    }

  // pressure at sea level 
  if (r.mask & MMASK_PRES)
    {
      s.press.tiempo += intervalo;
      s.press.acum += (r.press * intervalo);
      if (s.press.n == 0 || s.press.vmax < r.press)
        {
          s.press.vmax = r.press;
          s.press.tvmax = r.datime;
        }
      if (s.press.n == 0 || s.press.vmin > r.press)
        {
          s.press.vmin = r.press;
          s.press.tvmin = r.datime;
        }
      (s.press.n)++;
    }

  // wspd
  if (r.mask & MMASK_WSPD)
    {
      s.wspd.tiempo += intervalo;
      s.wspd.acum += (r.wspd * intervalo);
      if (s.wspd.n == 0 || s.wspd.vmax < r.wspd)
        {
          s.wspd.vmax = r.wspd;
          s.wspd.tvmax = r.datime;
          if (r.mask & MASK_WDIR)
            s.wspd.aux = r.wdir;
        }
      (s.wspd.n)++;

      s.wgust.tiempo += intervalo;
      s.wgust.acum += (r.wgust * intervalo);
      if (s.wgust.n == 0 || s.wgust.vmax < r.wgust)
        {
          s.wgust.vmax = r.wgust;
          s.wgust.tvmax = r.datime;
          /* here s.wgust.aux is the direction of max */
          if (r.mask & MASK_WDIR)
            s.wgust.aux = r.wdir;
        }
      (s.wgust.n)++;
    }

  /* wind dir. Here we compute component X (E) and Component Y(N) */
  if ((r.mask & MMASK_WDIR) && (r.mask & MMASK_WSPD))
    {
      s.wdir.tiempo += intervalo;
      s.wdir.acum += (r.wspd * intervalo);
      s.wdir.vmax += (r.wspd * intervalo) * sin(r.wdir * GRAD2RAD);
      s.wdir.vmin += (r.wspd * intervalo) * cos(r.wdir * GRAD2RAD);
      (s.wdir.n)++;
    }

  /* Precipitation is different
    in acum we store the acumulate rain of intervalo
    in vmax we store the maximum intensity (mm/hour)
    in vmin we store the last acum rain
    in tvmin we store the last date with rain not 0.0
  */
  if (r.mask & MMASK_RAIN)
    {
      if (s.rain.n == 0)
        {
          // first value
          s.rain.acum = 0.0;
          s.rain.aux = r.rain;
          s.rain.vmax = 0.0;
          s.rain.taux = r.datime;
          (s.rain.n)++;
        }
      else
        {
          aux = r.rain - s.rain.aux;
          if ((aux >= 0.0)  && ((r.mask & MMASK_FAKERAIN) == 0))
            {
              s.rain.acum += aux;
              if (aux > 0.0 && (3600.0 * aux / (r.datime - s.rain.taux)) > s.rain.vmax)
                {
                  s.rain.vmax =  3600.0 * aux /(r.datime - s.rain.taux);
                  s.rain.tvmax = r.datime;
                }
              s.rain.taux = r.datime;
            }
          s.rain.aux = r.rain;
          s.rain.tiempo += intervalo;
          (s.rain.n)++;
        }
    }

  // at the end update global mask and n
  s.mask |= r.mask;
  s.latest = r.datime;
  (s.n)++;

  return 1;
}

int interp_met_report(met_report& r, struct obs_data *o, size_t nobs, time_t datime,
                met_station& e)
{
  double tx;
  size_t i = 0;
  struct met_report r1, r2;

  while (o[i].datime < datime && i < nobs) i++;

  if (i < nobs && o[i].datime == datime)
    {
      r = met_report(&o[i], e);
      return 1;
    }

  if (i < nobs && i > 0)
    {
      r1 = met_report(&o[i - 1], e);
      r2 = met_report(&o[i], e);

      r.mask = (r1.mask & r2.mask);
      if ((r.mask & MMASK_DATIME) == 0)
        return 0;
      tx = ((double)datime - r1.datime) / ((double)r2.datime - r1.datime);
      r.datime = datime;
      r.interval = datime - r1.datime;

      if (r.mask & MMASK_HUM_INT)
        r.hum_int = r1.hum_int + (r2.hum_int - r1.hum_int) * tx;

      if (r.mask & MMASK_HUM_EXT)
        r.hum_out = r1.hum_out + (r2.hum_out - r1.hum_out) * tx;

      if (r.mask & MMASK_TEMP_INT)
        r.temp_int = r1.temp_int + (r2.temp_int - r1.temp_int) * tx;

      if (r.mask & MMASK_TEMP_EXT)
        r.temp_out = r1.temp_out + (r2.temp_out - r1.temp_out) * tx;

      if ((r.mask & MMASK_HUM_INT) && (r.mask & MMASK_TEMP_INT))
        r.tdew_int = r1.tdew_int + (r2.tdew_int - r1.tdew_int) * tx;

      if ((r.mask & MMASK_HUM_EXT) && (r.mask & MMASK_TEMP_EXT))
        r.tdew_out = r1.tdew_out + (r2.tdew_out - r1.tdew_out) * tx;

      if (r.mask & MMASK_PRES)
        {
          r.sea_press = r1.sea_press + (r2.sea_press - r1.sea_press) * tx;
          r.press = r1.press + (r2.press - r1.press) * tx;
        }
      if (r.mask & MMASK_WSPD)
        {
          r.wspd = r1.wspd + (r2.wspd - r1.wspd) * tx;
          r.wgust = r1.wgust + (r2.wgust - r1.wgust) * tx;
        }

      if (r.mask & MMASK_WDIR)
        r.wdir = r1.wdir + (r2.wdir - r1.wdir) * tx;

      if (r.mask & MMASK_RAIN)
        r.rain = r1.rain + (r2.rain - r1.rain) * tx;
    }
  else
    return 0;
  return 1;
}
