/* $Id$ */ 
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "metgra.h"


// Limpia un metgra_report
metgra_report::metgra_report()
    {
        memset(this, 0, sizeof(class metgra_report));
    };

// inicialización por defecto
metgra::metgra() 
{
   memset (this, 0, sizeof(class metgra));
};

// destructor de metgra
metgra::~metgra()
{
   if (dat != NULL && ndat >= 0)
      delete [] dat;
   if (im != NULL)
      gdImageDestroy(im);
};

// set titulo
int metgra::set_title(const char * source)
{
   if (strlen(source) >= 128)
     return 0;
   strcpy( title, source);
   return 1;
};

// set subtit
int metgra::set_subtit(const char * source)
{
   if (strlen(source) >= 128)
     return 0;
   strcpy( subtit, source);
   return 1;
};

// set file
int metgra::set_file(const char *source)
{
   if (strlen(source) >= 256)
     return 0;
   strcpy( file, source);
   return 1;
};

// set astronomic data 
int metgra::set_astr(const time_t ref, const double lat, const double lon)
{
  int i, j, k = 0, oca, loca;
  time_t tt, toca[2];
  struct astro a;
  struct tm ti;

  gmtime_r(&ref, &ti);

  loca = (metgra_ortocaso2( lat, lon, &ti, &a) == 0) ? 0 : 1;
  j = loca;

  for (i = 0; i <= 86400; i += 120)
    {
      tt = ref + i;
      gmtime_r(&tt, &ti);
      oca = (metgra_ortocaso2( lat, lon, &ti, &a) == 0) ? 0 : 1;
      if( oca != loca)
        toca[k++] = tt;
      loca = oca;
    }

  if ( k < 2)
    return 0; // no sunrise / sunset found

  if (j == 0)
    {
      ocaso = toca[0];
      orto = toca[1];
    }
  else
    {
      ocaso = toca[1];
      orto = toca[0];
    }
   
   return 1;
}

/*!
  \fn int metgra::coord_x(time_t t)
  \brief obtiene la coordenada x en pixels (tiempo) respecto limite izquierdo de la gráfica 
  \param t time_t del tiempo solicitado
*/
int metgra::coord_x(time_t t)
{
   int x;

   if(t <= tbegin)
      return mx;

   x = (t - tbegin) / secppix;
   if (x < (sx - 2 * mx))
     return (mx + x);
   else
     return (sx - mx);
}

/*!
  \fn metgra::int coord_y(double d)
  \brief obtiene la coordenada y (en pixels) respecto del limite superior del grafico
  \param d dato del que se quiere obtener su coordenada
*/
int metgra::coord_y(double d)
{
  int iy;
  double y;

  if (et == eb)
    return METEO_FAILURE;

  y = ((et - d) * dsy) / (et - eb);
  iy = (int)(y + 0.5);

  if (iy < 0)
    iy = 0;
  else if (iy >= dsy)
    iy = dsy - 1;

  return iy + myt;
}

