/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor   *
 *   gbv@oxixares.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef METGRA_H

#include <iostream>
#include <string>
#include <cstring>
#include <exception>
#include <cmath>
#include <cstdlib>

#include <time.h>
#include <sys/stat.h>
#include <dirent.h>

// Libreria gráfica
#include "gd.h"

/* conversiones utiles */
/*! \def GRA2RAD
    \brief Factor de conversión de grados a radianes
*/
#ifndef GRA2RAD
#define GRA2RAD (M_PI / 180.0)
#endif

/*! \def RAD2GRA
    \brief Factor de conversión de radianes a grados
*/
#ifndef RAD2GRA
#define RAD2GRA ( 180.0 / M_PI)
#endif

#ifndef METEO_SUCCESS
# define METEO_SUCCESS 1
#endif

#ifndef METEO_FAILURE
# define METEO_FAILURE 0
#endif



#define MASK_GRA_TEMP_EXT 1
#define MASK_GRA_TEMP_INT 2
#define MASK_GRA_HUM_EXT 4
#define MASK_GRA_HUM_INT 8
#define MASK_GRA_WSPD 16
#define MASK_GRA_WDIR 32
#define MASK_GRA_RAIN 64
#define MASK_GRA_PRES 128
#define MASK_GRA_SEA_PRES 256

#define DMASK_HUM_INT 1
#define DMASK_HUM_EXT 2
#define DMASK_TEMP_INT 4
#define DMASK_TEMP_EXT 8
#define DMASK_PRES 16
#define DMASK_WSPD 32
#define DMASK_WDIR 64
#define DMASK_RAIN 128
#define DMASK_INTERV 256
#define DMASK_DATIME 512
#define DMASK_PREC MMASK_RAIN
#define DMASK_WWSR 1023

#define DMASK_FAKERAIN 1024



using namespace std;

/*!
    \struct astro
   \brief Coordenadas del sol para las rutinas astro
*/
struct astro{
   double L; /* longitud ecliptica */
   double RA; /* ascension recta */
   double h;
   double gmst0;
   double delta;
   double obliq;
   double x;
   double y;
   double z;
};

/*!
  \class metgra_report 
  \brief estructura con datos procesados y preparados para las graficas
*/
class metgra_report {
    public:
    int mask;
    time_t datime;
    int interval;
    double temp_int;
    double temp_out;
    double hum_int;
    double hum_out;
    double tdew_int;
    double tdew_out;
    double press;
    double sea_press;
    double wspd;
    double wdir;
    double wgust;
    double wdgust;
    double rain;

    metgra_report();
};


/*!
  \class emag_dat
  \brief contiene toda la informacion necesaria para plotear un grafico
*/
class metgra {
   public:
   int mask; /*!< entero con la máscara de la variable a pintar */
   char title[128]; /*!< Titulo a imprimir en la cabecera del grafico */
   char subtit[128]; /*!< cadena con el subtitulo, normalmente el nombre de variables */
   char file[256]; /*!< Nombre del path completo del fichero a abrir */
   char font[256]; /*!< Path con el fichero truefont a utilizar */
   time_t tbegin; /*!<  Unix time del instante correspondiente al limite izquierdo de la grafica */
   time_t tend; /*!<  Unix time del instante correspondiente al limite derecho de la grafica */
   time_t t0; /*!< Unix time del instante a que corresponde el primer dato */
   time_t orto; /*!< Unix time del instante aproximado del orto */
   time_t ocaso; /*!< Unix time del instante aproximado del ocaso */
   struct metgra_report *dat; /*!< puntero array de datos en formato struct \ref metgra_report */
   int ndat; /*!< numero de datos con contenido del array */
   int sx; /*!< tamaño en pixels coordenada x (horizontal) de todo el grafico */
   int sy; /*!< tamaño en pixels coordenada y (vertical) de todo el grafico */
   int dsx; /*!< tamaño en pixels coordenada x del area de datos */ 
   int dsy; /*!< tamaño en pixels coordenada y del area de datos */ 
   int mx; /*!< tamaño en pixels de cada uno de los margenes laterales */
   int myt; /*!< tamaño en pixels del márgen superior */
   int myb; /*!< tamaño en pixels del márgen inferior */
   double et; /*!< valor del dato en el limite superior del área de datos */
   double eb; /*!< valor del dato en el límite inferior del area de datos */ 
   int secppix; /*!< Seconds per_pixel */
   int secpvl; /*!< Seconds per vertical line */
   double datppix; /*!< Data increment per vertical pixel */
   gdImagePtr im; /*!< puntero a imagen grafica gd */
 

   metgra();

  ~metgra();

   // functions
   int set_title(const char *s);
   int set_subtit(const char *s);
   int set_file(const char *s);
   int set_astr(const time_t tref, const double lat, const double lon);
   int crea_img();
   int destroy_img();
   int coord_x(time_t t);
   int coord_y(double d);
   int plot_graf();
   int pinta_noche();
   int pinta_noche2();
   int fondo_clasic_temp();
   int fondo_clasic_hum();
   int fondo_clasic_dirw();
   int fondo_clasic_wspd();
   int fondo_clasic_press();
   int fondo_clasic_prec();
};

int metgra_sunpos(double *azim, double *altit, struct tm *hora, struct astro *a, double lat, double lon, int mode);
int metgra_ortocaso(double latit, double longit, struct tm *hora0, struct astro *a);
int metgra_ortocaso2(double latit, double longit, struct tm *hora0, struct astro *a);
char * get_fontpath(void);
char * get_dirtmp(void);

extern char FONT_DEBIAN[];
extern char FONT_CENTOS[];            
extern char FONT_OPENSUSE[];       
extern char copyright[];
extern char DIR_TMP_DEBIAN[];
extern char DIR_TMP_SUSE[];

#endif // METGRA_H