/* $Id$ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "metgra.h"

char FONT_DEBIAN[] = "/usr/share/fonts/truetype/freefont/FreeSans.ttf";
char FONT_CENTOS[] = "/usr/share/X11/fonts/TTF/luxisr.ttf";
char FONT_OPENSUSE[] = "/usr/share/fonts/truetype/FreeSans.ttf";
char copyright[]= "(c) METGRA library (GPL v.3)";

char DIR_TMP_DEBIAN[] = "/var/www/tmp";
char DIR_TMP_SUSE[] = "/srv/www/htdocs/tmp";
char DIR_TMP_CENTOS[] = "/var/www/html/tmp";

/*!
  \fn int crea_img(int sx, int sy)
  \brief Crea una imagen base
  \param im puntero a gdImagePtr donde se crea la imagen gd
  \param sx tamaño horizontal en pixels
  \param sy tamaño vertical en pixles

   Devuelve \ref METEO_SUCCESS si la imagen ha sido creada con exito,
   en caso contrario devuelve \ref METEO_FAILURE
*/
int metgra::crea_img()
{
  /* control de argumentos */
  if (im != NULL || sx <= 0 || sx > 1800 || sy <= 0 || sy > 1800)
    return METEO_FAILURE;

  /* creacion de imagen gd */
  if ((im = gdImageCreateTrueColor(sx, sy)) != NULL)
    {
      return METEO_SUCCESS;
    }

  return METEO_FAILURE;
}


/*!
  \fn int destroy_img()
  \brief destruye una imagen base
  \param im puntero a gdImagePtr a destruir

   Devuelve \ref METEO_SUCCESS si la imagen se ha destruido con exito,
   en caso contrario devuelve \ref METEO_FAILURE
*/
int metgra::destroy_img()
{
  // control de argumentos
  if (im == NULL)
    return METEO_FAILURE;

  // destrucción de imagen gd
  gdImageDestroy(im);
  im = NULL; // this will said it is destroyed
  return METEO_SUCCESS;
}


/*!
  \fn int metgra::plot_graf()
  \brief Common routine to plot a graphic
*/
int metgra::plot_graf()
{
  int titulo, csubtit, cdato, cdato2, ldato, ldato1, ldato2, brect[8], papel, dx, dy;
  int xx0, xx1, yy0, yy1, i, j;
  char fechas[128], fecha1[32], fecha2[32];
  struct tm tmb, tme;

  gdImagePtr imb;
  FILE *f;

  // crea la imagen
  if ( crea_img() == METEO_FAILURE)
    {
      throw("No se ha podido crear la imagen gd");
    }

  papel = gdImageColorAllocate(im, 255, 255, 255); /* Color de fondo del papel */
  titulo = gdImageColorAllocate(im, 0, 80, 0); /* Color titulo */
  csubtit = gdImageColorAllocate(im, 0, 80, 90); /* Color subtitulo */
  cdato = gdImageColorAllocate(im, 0, 0, 90); // color de curva de temperatura 
  cdato2 = gdImageColorAllocate(im, 140, 0, 0); // color de viento maximo
  imb = gdImageCreate(2, 2);
  ldato = gdImageColorAllocate(imb, 0, 0, 90); /* Color de curva de temperatura */
  ldato1 = gdImageColorAllocate(imb, 0,100, 0); /* Color de curva de punto de rocio */
  ldato2 = gdImageColorAllocate(imb, 100, 0, 0); // Color de viento maximo
  gdImageSetBrush(im, imb);

  // pinta el fondo
  if ((mask & MASK_GRA_TEMP_EXT) || (mask & MASK_GRA_TEMP_INT))
    {
      fondo_clasic_temp();
    }
  else if ((mask & MASK_GRA_HUM_EXT) || (mask & MASK_GRA_HUM_INT))
   {
      fondo_clasic_hum();
   }
  else if (mask & MASK_GRA_RAIN)
   {
      fondo_clasic_prec();
   }
  else if (mask & MASK_GRA_WDIR)
   {
      fondo_clasic_dirw();
   }
  else if (mask & MASK_GRA_WSPD)
   {
      fondo_clasic_wspd();
   }
  else if ((mask & MASK_GRA_PRES) || (mask & MASK_GRA_SEA_PRES))
   {
      fondo_clasic_press();
   }

  // Escritura del titulo en parte superior de grafica
  gdImageStringFT(NULL, &brect[0], titulo, font, 9, 0, sx / 2, 7 + dy, title);
  dx = (brect[6] - brect[2]) / 2;
  dy = (brect[3] - brect[7]) / 2;
  gdImageStringFT(im, &brect[0], titulo, font, 9, 0, sx / 2 + dx, 7 + dy, title);

  // Escritura del Subtitulo
  gdImageStringFT(NULL, &brect[0], csubtit, font, 9, 0, sx / 2, 19 + dy, subtit);
  dx = (brect[6] - brect[2]) / 2;
  dy = (brect[3] - brect[7]) / 2;
  gdImageStringFT(im, &brect[0], csubtit, font, 9, 0, sx / 2 + dx, 19 + dy, subtit);

  // Fecha y hora del primer y ultimo dato
  gmtime_r(&(dat[0].datime), &tmb);
  strftime(fecha1,32,"%d/%m/%Y %H:%M", &tmb); 
  gmtime_r(&(dat[ndat - 1].datime), &tme);
  strftime(fecha2,32,"%d/%m/%Y %H:%M", &tme); 
  sprintf(fechas,"Datos desde %s hasta %s UTC", fecha1, fecha2);
  gdImageStringFT(NULL, &brect[0], cdato, font, 9, 0, sx / 2, 33 + dy, fechas);
  dx = (brect[6] - brect[2]) / 2;
  dy = (brect[3] - brect[7]) / 2;
  gdImageStringFT(im, &brect[0], cdato, font, 9, 0, sx / 2 + dx, 33 + dy, fechas);

  // escritura de los datos
  if (mask & MASK_GRA_TEMP_EXT)
    {
      // temperatura
      gdImageFilledRectangle(imb, 0, 0, 1, 1, ldato);
      for ( i = 0, j = 0; i < ndat; i++)
        {
          if (dat[i].mask & DMASK_TEMP_EXT)
            {
              xx1 = coord_x( dat[i].datime);
              yy1 = coord_y( dat[i].temp_out);
              if (j)
                {
                  gdImageLine( im, xx0, yy0, xx1, yy1, gdBrushed);
                }
              xx0 = xx1;
              yy0 = yy1;
              j++;
            }
          else
            j = 0;
        }

      // Punto de rocio
      gdImageFilledRectangle(imb, 0, 0, 1, 1, ldato1);
      for ( i = 0, j = 0; i < ndat; i++)
        {
          if ((dat[i].mask & DMASK_TEMP_EXT) && (dat[i].mask & DMASK_HUM_EXT))
            {
              xx1 = coord_x( dat[i].datime);
              yy1 = coord_y( dat[i].tdew_out);
              if (j)
                {
                  gdImageLine( im, xx0, yy0, xx1, yy1, gdBrushed);
                }
              xx0 = xx1;
              yy0 = yy1;
              j++;
            }
          else
            j = 0;
        }
    }
  else if (mask & MASK_GRA_TEMP_INT)
    {
      // temperatura interna
      gdImageFilledRectangle(imb, 0, 0, 1, 1, ldato);
      for ( i = 0, j = 0; i < ndat; i++)
        {
          if (dat[i].mask & DMASK_TEMP_INT)
            {
              xx1 = coord_x( dat[i].datime);
              yy1 = coord_y( dat[i].temp_int);
              if (j)
                {
                  gdImageLine( im, xx0, yy0, xx1, yy1, gdBrushed);
                }
              xx0 = xx1;
              yy0 = yy1;
              j++;
            }
          else
            j = 0;
        }

      // Punto de rocio
      gdImageFilledRectangle(imb, 0, 0, 1, 1, ldato1);
      for ( i = 0, j = 0; i < ndat; i++)
        {
          if ((dat[i].mask & DMASK_TEMP_INT) && (dat[i].mask & DMASK_HUM_INT))
            {
              xx1 = coord_x( dat[i].datime);
              yy1 = coord_y( dat[i].tdew_int);
              if (j)
                {
                  gdImageLine( im, xx0, yy0, xx1, yy1, gdBrushed);
                }
              xx0 = xx1;
              yy0 = yy1;
              j++;
            }
          else
            j = 0;
        }
    }
  else if (mask & MASK_GRA_HUM_EXT)
    {
      // humedad
      gdImageFilledRectangle(imb, 0, 0, 1, 1, ldato1);
      for ( i = 0, j = 0; i < ndat; i++)
        {
          if (dat[i].mask & DMASK_HUM_EXT)
            {
              xx1 = coord_x( dat[i].datime);
              yy1 = coord_y( dat[i].hum_out);
              if (j)
                {
                  gdImageLine( im, xx0, yy0, xx1, yy1, gdBrushed);
                }
              xx0 = xx1;
              yy0 = yy1;
              j++;
            }
          else
            j = 0;
        }
    }
  else if (mask & MASK_GRA_HUM_INT)
    {
      // humedad
      gdImageFilledRectangle(imb, 0, 0, 1, 1, ldato1);
      for ( i = 0, j = 0; i < ndat; i++)
        {
          if (dat[i].mask & DMASK_HUM_INT)
            {
              xx1 = coord_x( dat[i].datime);
              yy1 = coord_y( dat[i].hum_int);
              if (j)
                {
                  gdImageLine( im, xx0, yy0, xx1, yy1, gdBrushed);
                }
              xx0 = xx1;
              yy0 = yy1;
              j++;
            }
          else
            j = 0;
        }
    }
  else if (mask & MASK_GRA_RAIN)
    {
      // precipitación acumulada
      double acum = 0.0, acum0 = 0.0;
      int yy2, xx2,  iprec = 0;
      gdImageFilledRectangle(imb, 0, 0, 1, 1, ldato);
      for ( i = 0, j = 0; i < ndat; i++)
        {
          if (dat[i].mask & DMASK_RAIN)
            {
              if ((dat[i].mask & DMASK_FAKERAIN) == 0)
              {
                 if (j)
                   acum += ((dat[i].rain) - acum0);
                 else
                 {
                   acum0 = dat[i].rain;
                   acum = 0.0;
                 }
              }
              if (acum)
                iprec = 1;
              xx1 = coord_x( dat[i].datime);
              while (acum > 10.0)
              {
                 // vaciado de pluviografo
                 xx2 = xx0 + (xx1 - xx0) * (10.0 - acum0) / (acum - acum0);
                 gdImageLine( im, xx0, yy0, xx2, myt, gdBrushed);
                 gdImageLine( im, xx2, myt, xx2, myt + dsy, gdBrushed);
                 acum -= 10.0;
                 acum0 += 10.0;
                 xx0 = xx2;
                 yy0 = myt + dsy;
              }

              yy1 = coord_y(acum);
              if (j)
                {
                  gdImageLine( im, xx0, yy0, xx1, yy1, gdBrushed);
                }
              xx0 = xx1;
              yy0 = yy1;
              acum0 = dat[i].rain;
              j++;
            }
          else
            j = 0;
        }

        // si no hay precipitación se indica en la grafica
        if (iprec == 0)
        {
          xx0 = sx / 2;
          yy0 = coord_y (5.2);
          gdImageStringFT(NULL, &brect[0], cdato, font, 10, 0, xx0, yy0, "La precipitación acumulada es 0.0 mm");
          dx = (brect[6] - brect[2]) / 2;
          dy = (brect[3] - brect[7]) / 2;
          gdImageFilledRectangle(im, brect[6] + dx, brect[7] + dy , brect[2] + dx , brect[3] + dy, papel);
          gdImageStringFT(im, &brect[0], cdato, font, 10, 0, xx0 + dx, yy0 + dy, "La precipitación acumulada es 0.0 mm");
        }
    }
   else if (mask & MASK_GRA_WDIR)
    {
      // dirección del viento
      int yyt, yyb, xx2;

      yyt = coord_y( 270.0);
      yyb = coord_y( 90.0);

      gdImageSetAntiAliased(im, cdato);
      for ( i = 0, j = 0; i < ndat; i++)
        {
          if ((dat[i].mask & DMASK_WDIR) && (dat[i].mask & DMASK_WSPD))
            {
              xx1 = coord_x( dat[i].datime);
              yy1 = coord_y( dat[i].wdir);

              if (j && dat[i].wgust == 0.0)
              {
                 yy1 = yy0;
              }

              if (j)
                {
                  if ((yy0 >= yyb) && (yy1 <= yyt))
                  {
                     xx2 = (xx1 + xx0) / 2;
                     gdImageLine( im, xx0, yy0, xx2, myt + dsy, gdAntiAliased);
                     gdImageLine( im, xx2, myt, xx1, yy1, gdAntiAliased);
                  }
                  else if ( (yy0 <= yyt) && (yy1 >= yyb))
                  {
                     xx2 = (xx1 + xx0)/2;
                     gdImageLine( im, xx0, yy0, xx2, myt, gdAntiAliased);
                     gdImageLine( im, xx2, myt + dsy, xx1, yy1, gdAntiAliased);
                  }
                  else 
                    gdImageLine( im, xx0, yy0, xx1, yy1, gdAntiAliased);
                }
              xx0 = xx1;
              yy0 = yy1;
              j++;
            }
          else
            j = 0;
        }
     }
  else if (mask & MASK_GRA_WSPD)
    {
      // racha
      gdImageSetAntiAliased(im, cdato2);
      gdImageFilledRectangle(imb, 0, 0, 1, 1, ldato2);
      for ( i = 0, j = 0; i < ndat; i++)
        {
          if (dat[i].mask & DMASK_WSPD)
            {
              xx1 = coord_x( dat[i].datime);
              yy1 = coord_y( dat[i].wgust);
              if (j)
                {
                  gdImageLine( im, xx0, yy0, xx1, yy1, gdAntiAliased);
                }
              xx0 = xx1;
              yy0 = yy1;
              j++;
            }
          else
            j = 0;
        }

      // medio
      gdImageSetAntiAliased(im, cdato);
      for ( i = 0, j = 0; i < ndat; i++)
        {
          if (dat[i].mask & DMASK_WSPD)
            {
              xx1 = coord_x( dat[i].datime);
              yy1 = coord_y( dat[i].wspd);
              if (j)
                {
                  gdImageLine( im, xx0, yy0, xx1, yy1, gdAntiAliased);
                }
              xx0 = xx1;
              yy0 = yy1;
              j++;
            }
          else
            j = 0;
        }
     }
  else if (mask & MASK_GRA_PRES)
    {
      // presion al nivel de la estación
      gdImageFilledRectangle(imb, 0, 0, 1, 1, ldato1);
      for ( i = 0, j = 0; i < ndat; i++)
        {
          if (dat[i].mask & DMASK_PRES)
            {
              xx1 = coord_x( dat[i].datime);
              yy1 = coord_y( dat[i].press);
              if (j)
                {
                  gdImageLine( im, xx0, yy0, xx1, yy1, gdBrushed);
                }
              xx0 = xx1;
              yy0 = yy1;
              j++;
            }
          else
            j = 0;
        }
    }
  else if (mask & MASK_GRA_SEA_PRES)
    {
      // presion al nivel de la estación
      gdImageFilledRectangle(imb, 0, 0, 1, 1, ldato1);
      for ( i = 0, j = 0; i < ndat; i++)
        {
          if (dat[i].mask & DMASK_PRES)
            {
              xx1 = coord_x( dat[i].datime);
              yy1 = coord_y( dat[i].sea_press);
              if (j)
                {
                  gdImageLine( im, xx0, yy0, xx1, yy1, gdBrushed);
                }
              xx0 = xx1;
              yy0 = yy1;
              j++;
            }
          else
            j = 0;
        }
    }

  // saca el fichero
  if ((f = fopen(file,"w")) == NULL)
    {
      gdImageDestroy (imb);
      destroy_img();
      throw ("no se ha podido abrir el fichero grafico");
      return METEO_SUCCESS;
    }


  // escribe el grafico en el fichero
  gdImagePng(im, f);

  // Cierra el fichero
  fclose(f);

  // destruye la imagen
  gdImageDestroy (imb);
  destroy_img();
  return METEO_SUCCESS;

}


/*!
  \fn char * get_fontpath(void)
  \brief Get the trutypoe font file path
*/
char * get_fontpath(void)
{
  struct stat buf;

  /* Caso openSUSE */
  if (stat(FONT_OPENSUSE, &buf) == 0)
    return FONT_OPENSUSE;
  if (stat(FONT_CENTOS, &buf) == 0)
    return FONT_CENTOS;
  if (stat(FONT_DEBIAN, &buf) == 0)
    return FONT_DEBIAN;
  return NULL;
}

/*!
  \fn char * get_dirtmp(void)
  \brief Get the path where to set the graphic file. Then it is readed and served by httpd
*/
char * get_dirtmp(void)
{
  DIR * dd;

  /* Caso openSUSE */
  if ((dd = opendir(DIR_TMP_DEBIAN)) != NULL)
    return DIR_TMP_DEBIAN;
  if ((dd = opendir(DIR_TMP_SUSE)) != NULL)
    return DIR_TMP_SUSE;
  if ((dd = opendir(DIR_TMP_CENTOS)) != NULL)
    return DIR_TMP_SUSE;
  return NULL;
}



/*!
  \fn int metgra::pinta_noche()
  \brief Plots the night zone in a graphic
*/
int metgra::pinta_noche()
{
  int noche, xorto, xocaso;
  time_t tort, toca;

  /* color de noche */
  //noche = gdImageColorAllocate (im, 245, 230, 210);
  noche = gdImageColorAllocate (im, 210, 230, 245);

  xorto = coord_x(orto);
  xocaso = coord_x(ocaso);

  if (orto < ocaso)
    {
      gdImageFilledRectangle( im, mx, myt, xorto, myt + dsy, noche);
      tort = orto + 86400;
      toca = ocaso;
    }
  else
    {
      toca = ocaso;
      tort = orto;
    }

  while (toca < tend)
    {
       xorto = coord_x(tort);
       xocaso = coord_x(toca);

       if (tort < tend)
         gdImageFilledRectangle( im, xocaso, myt, xorto, myt + dsy, noche);
       else
         gdImageFilledRectangle( im, xocaso, myt, mx + dsx, myt + dsy, noche);

       toca += 86400;
       tort += 86400;
    }

  return METEO_SUCCESS;
}


