/* $Id: metgra_hum.cpp 114 2009-06-13 11:59:07Z gbv $ */ 
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*!
  \file metgra_hum.cpp
  \brief This file includes all rotines used to plot humidity data in metgra librry
*/
#include "metgra.h"

/*!
  \fn int metgra::fondo_clasic_temp()
  \brief Plots the backgroud of a temperature graphic in classical manner
*/
int metgra::fondo_clasic_hum()
{
  int fondo, linea, j;
  int xx0, xx1, yy0, yy1, brect[8], dx, dy;
  double aux;
  char caux[16], caux1[16];
  time_t taux;
  struct tm ti;

  fondo = gdImageColorAllocate( im, 255, 255, 255); // color del papel 
  //linea = gdImageColorAllocate( im, 235, 160, 120); // color lineas en papel 
  linea = gdImageColorAllocate( im, 120, 160, 235); // color lineas en papel 


  gdImageFilledRectangle(im, 0, 0, sx - 1, sy - 1, fondo);

  // pinta noche
  pinta_noche();

  gdImageRectangle(im, mx, myt, mx + dsx, myt + dsy, linea);

  // Lineas horizontales de 2 %
  for (aux = eb; aux <= et; aux += 2.0)
    {
      xx0 = coord_x(tbegin);
      xx1 = coord_x(tend);
      yy0 = coord_y(aux);
      yy1 = yy0;
      gdImageLine(im, xx0, yy0, xx1, yy1, linea);
    }

  // Lineas horizontales de 10% grados
  for (aux = eb; aux <= et; aux += 10.0)
    {
      xx0 = coord_x(tbegin);
      xx1 = coord_x(tend);
      yy0 = coord_y(aux);
      yy1 = yy0;
      gdImageLine(im, xx0, yy0 + 1, xx1, yy1 + 1 , linea);
    }

  // Lineas de tiempo
  for (taux = tbegin; taux < tend; taux += secpvl)
    {
      xx0 = coord_x(taux);
      xx1 = xx0;
      yy0 = myt;
      yy1 = myt + dsy;
      gdImageLine(im, xx0, yy0, xx1, yy1, linea);
      if ((taux % 21600) == 0)
        gdImageLine(im, xx0 + 1, yy0, xx1 + 1, yy1, linea);
    }

  // Valores de T en la gráfica
  taux = (tbegin / secpvl) * secpvl; 
  for (j = 1; taux < tend; taux += secpvl)
    {
      if ((taux % 21600) != 0 || j)
      {
         j = 0;
         continue;
      }

      j++;
      xx0 = coord_x(taux);
      for (aux = eb + 10.0; aux < et; aux += 10.0)
        {
          yy0 = coord_y(aux);
          sprintf(caux,"%.0lf", aux);

          /* Prueba el tamaño de la cadena a escribir */
          gdImageStringFT(NULL, &brect[0], linea, font, 8, 0, xx0, yy0, caux);
          dx = (brect[6] - brect[2]) / 2;
          dy = (brect[3] - brect[7]) / 2;
          /* pinta el cuadrado para que la imagen transparente quede bien */
          gdImageFilledRectangle(im, brect[6] + dx, brect[7] + dy , brect[2] + dx , brect[3] + dy, fondo);
          gdImageStringFT(im, &brect[0], linea, font, 8, 0, xx0 + dx, yy0 + dy, caux);
        }
    }

  // Hora en la grafica
  //  taux = ((tbegin + secpvl) / 21600 + 1) * 21600; 
  taux = (tbegin / secpvl) * secpvl; 
  for ( ; taux < tend; taux += secpvl)
    {
      if ((taux % 7200) != 0)
      {
        continue;
      }
      xx0 = coord_x(taux);
      yy0 = myt - 8;
      yy1 = myt + dsy + 6;
      gmtime_r(&taux, &ti);
      if ((taux % 21600) == 0)
        strftime( caux, 16, "%HZ", &ti);
      else
        strftime( caux, 16, "%H", &ti);

      /****** Parte superior ***********/
      /* Prueba el tamaño de la caadena a escribir */
      gdImageStringFT(NULL, &brect[0], linea, font, 8, 0, xx0, yy0, caux);

      dx = (brect[6] - brect[2]) / 2;
      dy = (brect[3] - brect[7]) / 2;
      /* pinta el cuadrado para que la imagen transparente quede bien */
      gdImageFilledRectangle(im, brect[6] + dx, brect[7] + dy , brect[2] + dx , brect[3] + dy, fondo);

      gdImageStringFT(im, &brect[0], linea, font, 8, 0, xx0 + dx, yy0 + dy, caux);

      /****** Parte inferior ***********/
      /* Prueba el tamaño de la caadena a escribir */
      gdImageStringFT(NULL, &brect[0], linea, font, 8, 0, xx0, yy1, caux);

      dx = (brect[6] - brect[2]) / 2;
      dy = (brect[3] - brect[7]) / 2;
      /* pinta el cuadrado para que la imagen transparente quede bien */
      gdImageFilledRectangle(im, brect[6] + dx, brect[7] + dy , brect[2] + dx , brect[3] + dy, fondo);

      gdImageStringFT(im, &brect[0], linea, font, 8, 0, xx0 + dx, yy1 + dy, caux);

      /* Pinta la fecha */
      if (ti.tm_hour == 12 && taux != tbegin)
        {
          yy1 = myt + 7;
          strftime( caux1, 16, "%d/%m/%Y", &ti);
          /* Prueba el tamaño de la caadena a escribir */
          gdImageStringFT(NULL, &brect[0], linea, font, 8, 0, xx0, yy1, caux1);

          dx = (brect[6] - brect[2]) / 2;
          dy = (brect[3] - brect[7]) / 2;
          /* pinta el cuadrado para que la imagen transparente quede bien */
          gdImageFilledRectangle(im, brect[6] + dx, brect[7] + dy , brect[2] + dx , brect[3] + dy, fondo);
          gdImageStringFT(im, &brect[0], linea, font, 8, 0, xx0 + dx, yy1 + dy, caux1);

          yy1 = myt + dsy - 8;
          strftime( caux1, 16, "%d/%m/%Y", &ti);
          /* Prueba el tamaño de la caadena a escribir */
          gdImageStringFT(NULL, &brect[0], linea, font, 8, 0, xx0, yy1, caux1);

          dx = (brect[6] - brect[2]) / 2;
          dy = (brect[3] - brect[7]) / 2;
          /* pinta el cuadrado para que la imagen transparente quede bien */
          gdImageFilledRectangle(im, brect[6] + dx, brect[7] + dy , brect[2] + dx , brect[3] + dy, fondo);
          gdImageStringFT(im, &brect[0], linea, font, 8, 0, xx0 + dx, yy1 + dy, caux1);
        }
    }

  // Leyenda de eje y
  xx0 = mx - 2;
  yy0 = myt + dsy / 2;
  gdImageStringFT(NULL, &brect[0], linea, font, 8, M_PI/2, xx0, yy0, "HUMEDAD RELATIVA (%)");
  dx = (brect[6] - brect[2]) / 2;
  dy = (brect[3] - brect[7]) / 2;
  gdImageStringFT(im, &brect[0], linea, font, 8, M_PI/2, xx0 + dx, yy0 - dy, "HUMEDAD RELATIVA (%)");

  // Eje y leyenda derecha
  xx0 = mx + dsx + 3;
  yy0 = myt + dsy / 2;
  gdImageStringFT(NULL, &brect[0], linea, font, 7, -(M_PI)/2, xx0, yy0, copyright);
  dx = (brect[6] - brect[2]) / 2;
  dy = (brect[3] - brect[7]) / 2;
  gdImageStringFT(im, &brect[0], linea, font, 7, -(M_PI)/2, xx0 + dx, yy0 - dy, copyright);

  return 1;
}
