/* $Id: wwsr_stats.c 98 2009-06-10 05:08:31Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor                       *
 *   gbv@oxixares.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*!
  \file wwsr_stats.c
  \brief This file include functions to extract climatic/statistics from a DB file with wwsr data
*/
#include "wwsr.h"


/*!
  \fn int summarize_realdata(struct stat_summary *s, const struct real_data *r)
  \brief Computes a struct \ref real_data into a struct \ref stat_summary
  \param s pointer to the target struct \ref stat_summary
  \param r pointer to a struct \ref real_summary with the data to be taken account

  It is assumed that the data are summarized chronologically 
  If any data has been computed, it returned 1, otherwise 0.
*/
int summarize_realdata(struct stat_summary *s, const struct real_data *r)
{
  double aux, interval;

  // Date and time must be in the interval
  if ((r->mask & MASK_DATIME) == 0)
    return 0;

  if ((r->datime > s->end) || (r->datime < s->begin))
    return 0;

  if (s->n == 0)
    interval = 0.0;
  else
    interval = r->datime - s->latest;

  // hum_int
  if (r->mask & MASK_HUM_INT)
    {
      s->hum_int.tiempo += interval;
      s->hum_int.acum += r->hum_int * interval;
      if (s->hum_int.n == 0 || s->hum_int.vmax < r->hum_int)
        {
          s->hum_int.vmax = r->hum_int;
          s->hum_int.tvmax = r->datime;
        }
      if (s->hum_int.n == 0 || s->hum_int.vmin > r->hum_int)
        {
          s->hum_int.vmin = r->hum_int;
          s->hum_int.tvmin = r->datime;
        }
      (s->hum_int.n)++;
    }

  // temp_int
  if (r->mask & MASK_TEMP_INT)
    {
      s->temp_int.tiempo += interval;
      s->temp_int.acum += (r->temp_int * interval);
      if (s->temp_int.n == 0 || s->temp_int.vmax < r->temp_int)
        {
          s->temp_int.vmax = r->temp_int;
          s->temp_int.tvmax = r->datime;
        }
      if (s->temp_int.n == 0 || s->temp_int.vmin > r->temp_int)
        {
          s->temp_int.vmin = r->temp_int;
          s->temp_int.tvmin = r->datime;
        }
      (s->temp_int.n)++;
    }

  // tdew_int
  if ((r->mask & MASK_TEMP_INT) && (r->mask & MASK_HUM_INT))
    {
      s->tdew_int.tiempo += interval;
      s->tdew_int.acum += (r->tdew_int * interval);
      if (s->tdew_int.n == 0 || s->tdew_int.vmax < r->tdew_int)
        {
          s->tdew_int.vmax = r->tdew_int;
          s->tdew_int.tvmax = r->datime;
        }
      if (s->tdew_int.n == 0 || s->tdew_int.vmin > r->tdew_int)
        {
          s->tdew_int.vmin = r->tdew_int;
          s->tdew_int.tvmin = r->datime;
        }
      (s->tdew_int.n)++;
    }

  // hum_out
  if (r->mask & MASK_HUM_EXT)
    {
      s->hum_out.tiempo += interval;
      s->hum_out.acum += (r->hum_out * interval);
      if (s->hum_out.n == 0 || s->hum_out.vmax < r->hum_out)
        {
          s->hum_out.vmax = r->hum_out;
          s->hum_out.tvmax = r->datime;
        }
      if (s->hum_out.n == 0 || s->hum_out.vmin > r->hum_out)
        {
          s->hum_out.vmin = r->hum_out;
          s->hum_out.tvmin = r->datime;
        }
      (s->hum_out.n)++;
    }

  // temp_out
  if (r->mask & MASK_TEMP_EXT)
    {
      s->temp_out.tiempo += interval;
      s->temp_out.acum += (r->temp_out * interval);
      if (s->temp_out.n == 0 || s->temp_out.vmax < r->temp_out)
        {
          s->temp_out.vmax = r->temp_out;
          s->temp_out.tvmax = r->datime;
        }
      if (s->temp_out.n == 0 || s->temp_out.vmin > r->temp_out)
        {
          s->temp_out.vmin = r->temp_out;
          s->temp_out.tvmin = r->datime;
        }
      (s->temp_out.n)++;
    }

  // tdew_out
  if ((r->mask & MASK_TEMP_EXT) && (r->mask & MASK_HUM_EXT))
    {
      s->tdew_out.tiempo += interval;
      s->tdew_out.acum += (r->tdew_out * interval);
      if (s->tdew_out.n == 0 || s->tdew_out.vmax < r->tdew_out)
        {
          s->tdew_out.vmax = r->tdew_out;
          s->tdew_out.tvmax = r->datime;
        }
      if (s->tdew_out.n == 0 || s->tdew_out.vmin > r->tdew_out)
        {
          s->tdew_out.vmin = r->tdew_out;
          s->tdew_out.tvmin = r->datime;
        }
      (s->tdew_out.n)++;
    }

  // pressure at station level 
  if (r->mask & MASK_PRES)
    {
      s->pres0.tiempo += interval;
      s->pres0.acum += (r->pres0 * interval);
      if (s->pres0.n == 0 || s->pres0.vmax < r->pres0)
        {
          s->pres0.vmax = r->pres0;
          s->pres0.tvmax = r->datime;
        }
      if (s->pres0.n == 0 || s->pres0.vmin > r->pres0)
        {
          s->pres0.vmin = r->pres0;
          s->pres0.tvmin = r->datime;
        }
      (s->pres0.n)++;
    }

  // pressure at sea level 
  if (r->mask & MASK_PRES)
    {
      s->press.tiempo += interval;
      s->press.acum += (r->press * interval);
      if (s->press.n == 0 || s->press.vmax < r->press)
        {
          s->press.vmax = r->press;
          s->press.tvmax = r->datime;
        }
      if (s->press.n == 0 || s->press.vmin > r->press)
        {
          s->press.vmin = r->press;
          s->press.tvmin = r->datime;
        }
      (s->press.n)++;
    }

  // wspd
  if (r->mask & MASK_WSPD)
    {
      s->wspd.tiempo += interval;
      s->wspd.acum += (r->wspd * interval);
      if (s->wspd.n == 0 || s->wspd.vmax < r->wspd)
        {
          s->wspd.vmax = r->wspd;
          s->wspd.tvmax = r->datime;
        }
      if (s->wspd.n == 0 || s->wspd.vmin > r->wspd)
        {
          s->wspd.vmin = r->wspd;
          s->wspd.tvmin = r->datime;
        }
      (s->wspd.n)++;

      s->wgust.tiempo += interval;
      s->wgust.acum += (r->wgust * interval);
      if (s->wgust.n == 0 || s->wgust.vmax < r->wgust)
        {
          s->wgust.vmax = r->wgust;
          s->wgust.tvmax = r->datime;
          /* here s->wgust.vmin is the direction of max */
          if (r->mask & MASK_WDIR)
            s->wgust.vmin = r->wdir;
        }
      (s->wgust.n)++;
    }

  /* wind dir. Here we compute component X (E) and Component Y(N) */
  if ((r->mask & MASK_WDIR) && (r->mask & MASK_WSPD))
    {
      s->wdir.tiempo += interval;
      s->wdir.acum += (r->wspd * interval);
      s->wdir.vmax += (r->wspd * interval) * sin(r->wdir * GRAD2RAD);
      s->wdir.vmin += (r->wspd * interval) * cos(r->wdir * GRAD2RAD);
      (s->wdir.n)++;
    }

  /* Precipitation is different
    in acum we store the acumulate prec of interval
    in vmax we store the maximum intensity (mm/hour)
    in vmin we store the last acum prec
    in tvmin we store the last date with prec not 0.0
  */
  if (r->mask & MASK_RAIN)
    {
      if (s->prec.n == 0)
        {
          // first value
          s->prec.acum = 0.0;
          s->prec.vmin = r->prec;
          s->prec.vmax = 0.0;
          s->prec.tvmin = r->datime;
          (s->prec.n)++;
        }
      else
        {
          aux = r->prec - s->prec.vmin;
          if ((aux >= 0.0) && ((r->mask & MASK_FAKERAIN) == 0))
            {
              s->prec.acum += aux;
              if (aux > 0.0 && (3600.0 * aux / (r->datime - s->prec.tvmin)) > s->prec.vmax)
                {
                  s->prec.vmax =  3600.0 * aux /(r->datime - s->prec.tvmin);
                  s->prec.tvmax = r->datime;
                }
              s->prec.tvmin = r->datime;
            }
          s->prec.vmin = r->prec;
          s->prec.tiempo += interval;
          (s->prec.n)++;
        }
    }

  // at the end update global mask and n
  s->mask |= r->mask;
  s->latest = r->datime;
  (s->n)++;

  return 1;
}

void print_stat_summary(struct stat_summary *s, FILE *f)
{
  struct tm t;
  char aux[32];

  if (s->mask & MASK_TEMP_EXT)
    {
      gmtime_r(&(s->temp_out.tvmax), &t);
      strftime(aux, 32, "%H:%M %d/%m/%Y", &t);
      fprintf( f, "TMAX =%6.1lfC (%s). ", s->temp_out.vmax, aux);
      gmtime_r(&(s->temp_out.tvmin), &t);
      strftime(aux, 32, "%H:%M %d/%m/%Y", &t);
      fprintf( f, "TMIN =%6.1lfC (%s). ", s->temp_out.vmin, aux);
      fprintf( f, "TMED =%6.1lfC ", s->temp_out.acum / s->temp_out.tiempo);
      fprintf( f, "VALID=%.1lf%%\n", 100.0 * s->temp_out.tiempo /(s->end - s->begin));
    }
  if (s->mask & MASK_HUM_EXT)
    {
      gmtime_r(&(s->hum_out.tvmax), &t);
      strftime(aux, 32, "%H:%M %d/%m/%Y", &t);
      fprintf( f, "HRMAX=%6.1lf%% (%s). ", s->hum_out.vmax, aux);
      gmtime_r(&(s->hum_out.tvmin), &t);
      strftime(aux, 32, "%H:%M %d/%m/%Y", &t);
      fprintf( f, "HRMIN=%6.1lf%% (%s). ", s->hum_out.vmin, aux);
      fprintf( f, "HRMED=%6.1lf%% ", s->hum_out.acum / s->hum_out.tiempo);
      fprintf( f, "VALID=%.1lf%%\n", 100.0 * s->hum_out.tiempo /(s->end - s->begin));
    }
  if ((s->mask & MASK_HUM_EXT) && (s->mask & MASK_TEMP_EXT))
    {
      gmtime_r(&(s->tdew_out.tvmax), &t);
      strftime(aux, 32, "%H:%M %d/%m/%Y", &t);
      fprintf( f, "TDMAX=%6.1lfC (%s). ", s->tdew_out.vmax, aux);
      gmtime_r(&(s->tdew_out.tvmin), &t);
      strftime(aux, 32, "%H:%M %d/%m/%Y", &t);
      fprintf( f, "TDMIN=%6.1lfC (%s). ", s->tdew_out.vmin, aux);
      fprintf( f, "TDMED=%6.1lfC ", s->tdew_out.acum / s->tdew_out.tiempo);
      fprintf( f, "VALID=%.1lf%%\n", 100.0 * s->tdew_out.tiempo / (s->end - s->begin));
    }
  if ((s->mask & MASK_WSPD))
    {
      gmtime_r(&(s->wgust.tvmax), &t);
      strftime(aux, 32, "%H:%M %d/%m/%Y", &t);
      fprintf( f, "GUST =%6.1lfm/s (%s). ", s->wgust.vmax, aux);
      fprintf( f, "VMED =%6.1lfm/s ", s->wspd.acum / s->wspd.tiempo);
      fprintf( f, "VALID=%.1lf%%\n", 100.0 * s->wspd.tiempo / (s->end - s->begin));
    }
  if (s->mask & MASK_PRES)
  {
      gmtime_r(&(s->press.tvmax), &t);
      strftime(aux, 32, "%H:%M %d/%m/%Y", &t);
      fprintf( f, "PMAX =%6.1lfHp (%s). ", s->press.vmax, aux);
      gmtime_r(&(s->press.tvmin), &t);
      strftime(aux, 32, "%H:%M %d/%m/%Y", &t);
      fprintf( f, "PMIN =%6.1lfHp (%s). ", s->press.vmin, aux);
      fprintf( f, "PMED =%6.1lfHp ", s->press.acum / s->press.tiempo);
      fprintf( f, "VALID=%.1lf%%\n", 100.0 * s->press.tiempo /(s->end - s->begin));
      gmtime_r(&(s->pres0.tvmax), &t);
      strftime(aux, 32, "%H:%M %d/%m/%Y", &t);
      fprintf( f, "P0MAX =%6.1lfHp (%s). ", s->pres0.vmax, aux);
      gmtime_r(&(s->pres0.tvmin), &t);
      strftime(aux, 32, "%H:%M %d/%m/%Y", &t);
      fprintf( f, "P0MIN =%6.1lfHp (%s). ", s->pres0.vmin, aux);
      fprintf( f, "P0MED =%6.1lfHp ", s->pres0.acum / s->pres0.tiempo);
      fprintf( f, "VALID=%.1lf%%\n", 100.0 * s->pres0.tiempo /(s->end - s->begin));
  }
  if (s->mask & MASK_PREC)
    {
      fprintf( f, "PREC =%6.1lf mm. ", s->prec.acum);
      gmtime_r(&(s->prec.tvmax), &t);
      strftime(aux, 32, "%H:%M %d/%m/%Y", &t);
      fprintf( f, "INT MAX.=%6.1lf mm/hr (%s) ", s->prec.vmax, aux);
      fprintf( f, "VALID=%.1lf%%\n", 100.0 * s->prec.tiempo / (s->end - s->begin));
    }
  
}

/* Print stats in a 'semicolon separated value' */
int print_stat_summary_csv(struct stat_summary *s, FILE *f, const char *sep, int viewmask)
{
   size_t l;
   struct tm t;
   char *c, saux[1024], aux[32];
   double x;

   c = saux;

   /* begin and end */
   gmtime_r(&(s->begin), &t);
   strftime(aux, 32, "%Y%m%d%H%M", &t);
   c += sprintf(c, "DINI=%s%s", aux, sep);
   gmtime_r(&(s->end), &t);
   strftime(aux, 32, "%Y%m%d%H%M", &t);
   c += sprintf(c, "DEND=%s%s", aux, sep);

   if (s->mask & viewmask & MASK_HUM_INT)
   {
      gmtime_r(&(s->hum_int.tvmax), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"HIX=%.1lf%sDHIX=%s%s",s->hum_int.vmax, sep, aux, sep);
      gmtime_r(&(s->hum_int.tvmin), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"HIN=%.1lf%sDHIN=%s%s",s->hum_int.vmin, sep, aux, sep);
      c += sprintf( c,"HIM=%.1lf%s",s->hum_int.acum /s->hum_int.tiempo, sep);
      c += sprintf( c,"HIV=%.1lf%s", 100.0 * s->hum_int.tiempo / (s->end - s->begin), sep);
   }

   if (s->mask & viewmask & MASK_TEMP_INT)
   {
      gmtime_r(&(s->temp_int.tvmax), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"TIX=%.1lf%sDTIX=%s%s",s->temp_int.vmax, sep, aux, sep);
      gmtime_r(&(s->temp_int.tvmin), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"TIN=%.1lf%sDTIN=%s%s",s->temp_int.vmin, sep, aux, sep);
      c += sprintf( c,"TIM=%.1lf%s",s->temp_int.acum /s->temp_int.tiempo, sep);
      c += sprintf( c,"TIV=%.1lf%s", 100.0 * s->temp_int.tiempo / (s->end - s->begin), sep);
   }

   if ((s->mask & viewmask & MASK_TEMP_INT ) && (s->mask & viewmask & MASK_HUM_INT))
   {
      gmtime_r(&(s->tdew_int.tvmax), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"TDIX=%.1lf%sDTDIX=%s%s",s->tdew_int.vmax, sep, aux, sep);
      gmtime_r(&(s->tdew_int.tvmin), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"TDIN=%.1lf%sDTDIN=%s%s",s->tdew_int.vmin, sep, aux, sep);
      c += sprintf( c,"TDIM=%.1lf%s",s->tdew_int.acum /s->tdew_int.tiempo, sep);
      c += sprintf( c,"TDIV=%.1lf%s", 100.0 * s->tdew_int.tiempo / (s->end - s->begin), sep);
   }


   if (s->mask & viewmask & MASK_HUM_EXT)
   {
      gmtime_r(&(s->hum_out.tvmax), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"HOX=%.1lf%sDHOX=%s%s",s->hum_out.vmax, sep, aux, sep);
      gmtime_r(&(s->hum_out.tvmin), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"HON=%.1lf%sDHON=%s%s",s->hum_out.vmin, sep, aux, sep);
      c += sprintf( c,"HOM=%.1lf%s",s->hum_out.acum /s->hum_out.tiempo, sep);
      c += sprintf( c,"HOV=%.1lf%s", 100.0 * s->hum_out.tiempo / (s->end - s->begin), sep);
   }

   if (s->mask & viewmask & MASK_TEMP_EXT)
   {
      gmtime_r(&(s->temp_out.tvmax), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"TOX=%.1lf%sDTOX=%s%s",s->temp_out.vmax, sep, aux, sep);
      gmtime_r(&(s->temp_out.tvmin), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"TON=%.1lf%sDTON=%s%s",s->temp_out.vmin, sep, aux, sep);
      c += sprintf( c,"TOM=%.1lf%s",s->temp_out.acum /s->temp_out.tiempo, sep);
      c += sprintf( c,"TOV=%.1lf%s", 100.0 * s->temp_out.tiempo / (s->end - s->begin), sep);
   }

   if ((s->mask & viewmask & MASK_TEMP_EXT ) && (s->mask & viewmask & MASK_HUM_EXT))
   {
      gmtime_r(&(s->tdew_out.tvmax), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"TDOX=%.1lf%sDTDOX=%s%s",s->tdew_out.vmax, sep, aux, sep);
      gmtime_r(&(s->tdew_out.tvmin), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"TDON=%.1lf%sDTDON=%s%s",s->tdew_out.vmin, sep, aux, sep);
      c += sprintf( c,"TDOM=%.1lf%s",s->tdew_out.acum /s->tdew_out.tiempo, sep);
      c += sprintf( c,"TDOV=%.1lf%s", 100.0 * s->tdew_out.tiempo / (s->end - s->begin), sep);
   }


   if (s->mask & viewmask & MASK_PRES)
   {
      gmtime_r(&(s->press.tvmax), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"PRX=%.1lf%sDPRX=%s%s",s->press.vmax, sep, aux, sep);
      gmtime_r(&(s->press.tvmin), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"PRN=%.1lf%sDPRN=%s%s",s->press.vmin, sep, aux, sep);
      c += sprintf( c,"PRM=%.1lf%s",s->press.acum /s->press.tiempo, sep);
      c += sprintf( c,"PRV=%.1lf%s", 100.0 * s->press.tiempo / (s->end - s->begin), sep);
      gmtime_r(&(s->pres0.tvmax), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"P0X=%.1lf%sDP0X=%s%s",s->pres0.vmax, sep, aux, sep);
      gmtime_r(&(s->pres0.tvmin), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"P0N=%.1lf%sDP0N=%s%s",s->pres0.vmin, sep, aux, sep);
      c += sprintf( c,"P0M=%.1lf%s",s->pres0.acum /s->pres0.tiempo, sep);
      c += sprintf( c,"P0V=%.1lf%s", 100.0 * s->pres0.tiempo / (s->end - s->begin), sep);
   }

   if (s->mask & viewmask & MASK_WSPD)
   {
      gmtime_r(&(s->wgust.tvmax), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"WSX=%.1lf%sDWSX=%s%s",s->wgust.vmax, sep, aux, sep);
      c += sprintf( c,"WDX=%.0lf%s",s->wgust.vmin, sep);
      c += sprintf( c,"WSM=%.1lf%s",s->wspd.acum /s->wspd.tiempo, sep);
      c += sprintf( c,"WSV=%.1lf%s", 100.0 * s->wspd.tiempo / (s->end - s->begin), sep);
      if (s->wdir.vmin != 0.0 || s->wdir.vmax != 0.0)
      {
         x = atan2(s->wdir.vmax, s->wdir.vmin) * RAD2GRAD;
         if (x < 0.0)
           x += 360.0;
         c += sprintf( c, "WDM=%.0lf%s", x, sep); 
      }
   }

   if (s->mask & viewmask & MASK_RAIN)
   {
      gmtime_r(&(s->prec.tvmax), &t);
      strftime(aux, 32, "%Y%m%d%H%M", &t);
      c += sprintf( c,"PCX=%.1lf%sDPCX=%s%s",s->prec.vmax, sep, aux, sep);
      c += sprintf( c,"PCT=%.1lf%s",s->prec.acum, sep);
      c += sprintf( c,"PCV=%.1lf%s", 100.0 * s->prec.tiempo / (s->end - s->begin), sep);
   }

   /* delete last ';' */ 
   if ((l = strlen(saux)))
     saux[ l - 1] = '\n';

   fprintf (f, "%s", saux);
   return 1;
}

