/* $Id: wwsr_tools.c 99 2009-06-11 20:55:03Z gbv $ */
/***************************************************************************
 *   Copyright (C) 2009 by Guillermo Ballester Valor   *
 *   gbv@oxixares.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/*!
  \file wwsr_tools.c
  \brief file to deal with library libmetcal
*/
#include "wwsr.h"

/*!
  \fn uint16_t read_uint16(uint8_t *bf)
  \brief read a two bytes unsigned int. First byte less significant
  \param bf unsigned byte array pointer
*/
uint16_t read_uint16(uint8_t *bf)
{
  return ((uint16_t)bf[0] + (uint16_t)bf[1] * 256);
}


/*!
  \fn int16_t read_sint16(uint8_t *bf)
  \brief read a two bytes signed int. First byte less significant
  \param bf unsigned byte array pointer
*/
int16_t read_sint16(uint8_t *bf)
{
  if (bf[1] & 0x80)
    return -((uint16_t)bf[0] + (uint16_t)(bf[1] & 0x7f) * 256);
  else
    return read_uint16(bf);
}

/*!
  \fn void print_bytes(char *bytes, int len)
  \brief print bytes for debug proposes
  \param bytes pointer to the array print
  \param len number of bytes to print
*/
void print_bytes(char *bytes, int len)
{
  int i;
  if (len > 0)
    {
      for (i=0; i<len; i++)
        {
          printf("%02x ", (int)((unsigned char)bytes[i]));
        }
      // printf("\"");
    }
}



/*!
  \fn unsigned int get_pos(uint16_t current, int nobs)
  \brief get the position of a observation
  \param current position of current data
  \param nobs observations before current data
*/
unsigned int get_pos(uint16_t current, int nobs)
{
  int pos;

  pos = (int)current - nobs * 16;
  if (pos < 256)
    pos = MEMSIZE - (256 - pos);
  return pos;
}



/* this version do not check for time */
void print_real_data(FILE *f, struct real_data *r,const char *sep)
{
  struct tm tim;
  char aux[32], *c, lin[256];

  memset(&tim, 0, sizeof(struct tm));
  gmtime_r(&(r->datime), &tim);
  strftime(aux, 32, "%Y%m%d%H%M", &tim);
  c = lin + sprintf(lin, "%s%s", aux, sep);
  c += sprintf(c, "%.0lf%s",  r->interval, sep);
  c += sprintf(c, "%.0lf%s", r->hum_int, sep);
  c += sprintf(c, "%.1lf%s", r->temp_int, sep);
  c += sprintf(c, "%.0lf%s", r->hum_out, sep);
  c += sprintf(c, "%.1lf%s", r->temp_out, sep);
  c += sprintf(c, "%.1lf%s", r->pres0, sep);
  c += sprintf(c, "%.1lf%s", r->press, sep);
  c += sprintf(c, "%.1lf%s", r->wspd, sep);
  c += sprintf(c, "%.0lf%s", r->wdir, sep);
  c += sprintf(c, "%.1lf%s", r->wgust, sep);
  c += sprintf(c, "%.1lf%s", r->prec, sep);
  *(c - 1) = '\n';
  fprintf( f, "%s", lin);
}

/* this version do not check for time */
void print_real_data_csv(FILE *f, struct real_data *r, const char *sep)
{
  struct tm tim;
  char aux[32], *c, lin[256];

  memset(&tim, 0, sizeof(struct tm));
  gmtime_r(&(r->datime), &tim);
  strftime(aux, 32, "%Y%m%d%H%M", &tim);
  c = lin + sprintf(lin, "DATE=%s%s", aux, sep);
  c += sprintf(c, "PER=%.0lf%s", r->interval, sep);
  c += sprintf(c, "HI=%.0lf%s", r->hum_int, sep);
  c += sprintf(c, "TI=%.1lf%s", r->temp_int, sep);
  c += sprintf(c, "TDI=%.1lf%s", r->tdew_int, sep);
  c += sprintf(c, "HO=%.0lf%s", r->hum_out, sep);
  c += sprintf(c, "TO=%.1lf%s", r->temp_out, sep);
  c += sprintf(c, "TDO=%.1lf%s", r->tdew_out, sep);
  c += sprintf(c, "P0=%.1lf%s", r->pres0, sep);
  c += sprintf(c, "PR=%.1lf%s", r->press, sep);
  c += sprintf(c, "WS=%.1lf%s", r->wspd, sep);
  c += sprintf(c, "WD=%.0lf%s", r->wdir, sep);
  c += sprintf(c, "WG=%.1lf%s", r->wgust, sep);
  *(c - 1) = '\n';
  fprintf( f, "%s", lin);
}


uint8_t bcdval( uint8_t a)
{
  return ((a >> 4) * 10) + (a & 0x0f);
}

/* returns the result of qflag, 0 is good. Otherwise we have problems */
int buf2obs (struct obs_data *o, unsigned char *buf)
{
  o->qflag = 0;
  o->interval = buf[0];
  if (o->interval > 60)
    o->qflag |= MASK_INTERV;

  o->hum_int = buf[1];
  if (o->hum_int < 5 || o->hum_int > 100)
    o->qflag |= MASK_HUM_INT;

  o->temp_int = read_sint16(&buf[2]);
  if (o->temp_int > 550 || o->temp_int < -350)
    o->qflag |= MASK_TEMP_INT;

  o->hum_out = buf[4];
  if (o->hum_out < 5 || o->hum_out > 100)
    o->qflag |= MASK_HUM_EXT;

  o->temp_out = read_sint16(&buf[5]);
  if (o->temp_out > 550 || o->temp_out < -350)
    o->qflag |= MASK_TEMP_EXT;

  o->press =  buf[7] + buf[8] * 256;
  if (o->press > 10650 || o->press < 5000)
    o->qflag |= MASK_PRES;

  o->wspd = buf[9];
  o->wgust = buf[10];
  o->wdir = buf[12];
  o->prec = buf[13] + buf[14] * 256;
  o->mask = buf[15];
  memcpy(&o->buf[0], buf, 16);
  return o->qflag;
}

void read_date(struct tm *tim, uint8_t *bf)
{
  time_t t;
  memset(tim, 0, sizeof(struct tm));
  tim->tm_year = (int)100 + bcdval(bf[0]);
  tim->tm_mon = bcdval(bf[1]) - 1;
  tim->tm_mday = bcdval(bf[2]);
  tim->tm_hour = bcdval(bf[3]);
  tim->tm_min = bcdval(bf[4]);
  tim->tm_sec = 0;
  tim->tm_isdst = -1;
  t = mktime(tim);
}

/*!
  \fn double value( double x, struct cal_pol *c)
  \brief returns the corrected value of a variable after calibration
  \param x input double. Is the value of the parameter readed by instrument
  \param c pointer to a struct \ref cal_pol where are stored the info needed to give the correct value
*/
double value( double x, struct cal_pol *c)
{
  return calibrated( x, c);
}


/*!
  \fn int obsdata_to_realdata(struct real_data *r, struct obs_data *o)
  \brief convert the data in a struct \ref obs_data and pass them to a struct \ref real_data
  \param r pointer to the target \ref real_data
  \param o pointer to source struct \ref obs_data

  Returns 1 if succeded, 0 otherwise
*/
int obsdata_to_realdata(struct real_data *r, struct obs_data *o, struct station_cal *s, double altitude)
{
  struct cal_pol *c;

  /* check if valid data */
  if ( o->mask == 0xFF || (o->hum_int == 0 && o->interval == 0))
    return 0;

  r->mask = 0;
  r->datime = o->datime;
  if (r->datime > 0)
    r->mask |= MASK_DATIME;

  r->interval = 60.0 * (unsigned int) o->interval;
  if (r->interval > 0.0)
    r->mask |= MASK_INTERV;

  r->hum_int = value((double) o->hum_int, find_cal_pol(s, r->datime, "hi"));
  if (r->hum_int > 5.0 && r->hum_int <= 100.0)
    r->mask |= MASK_HUM_INT;

  r->temp_int = value((double)(o->temp_int) * 0.1, find_cal_pol(s, r->datime, "ti"));
  if (r->temp_int > -15.0 && r->temp_int <= 50.0)
    r->mask |= MASK_TEMP_INT;

  if ((r->mask & MASK_TEMP_INT) && (r->mask & MASK_HUM_INT))
    {
      r->tdew_int = tdew(r->temp_int, r->hum_int);
    }

  r->hum_out = value((double) o->hum_out, find_cal_pol(s, r->datime, "ho"));
  if (r->hum_out > 5.0 && r->hum_out <= 100.0)
    r->mask |= MASK_HUM_EXT;

  r->temp_out = value((double)(o->temp_out) * 0.1, find_cal_pol(s, r->datime, "to"));
  if (r->temp_out > -35.0 && r->temp_out <= 50.0)
    r->mask |= MASK_TEMP_EXT;

  if ((r->mask & MASK_TEMP_EXT) && (r->mask & MASK_HUM_EXT))
    {
      r->tdew_out = tdew(r->temp_out, r->hum_out);
    }

  r->pres0 = value((double)(o->press) * 0.1, find_cal_pol(s, r->datime, "p"));
  r->press = r->pres0;

  if (altitude != 0.0)
    {
      if (r->mask & MASK_TEMP_EXT)
        r->press = psealevel(r->press, altitude, r->temp_out);
      else
        r->press = psealevel(r->press, altitude, r->temp_out - 0.0065 * altitude);
    }
  if (r->press < 1055.0 && r->press > 700.0)
    r->mask |= MASK_PRES;

  c = find_cal_pol(s, r->datime, "w");
  r->wspd = value((double)(o->wspd) * 0.36, c);
  r->wgust = value((double)(o->wgust) * 0.36, c);
  if (r->wspd >= 0.0 && r->wspd < 200.0 && r->wgust >= 0.0 && r->wgust < 200.0)
    r->mask |= MASK_WSPD;

  if (o->wdir > 16)
    r->wdir = 999.9;
  else
    r->wdir = 22.5 * (double) o->wdir;
  if (r->wdir == 0.0)
    r->wdir = 360.0;

  if (r->wdir >= 0.0 && r->wdir <= 360.0)
    r->mask |= MASK_WDIR;

  r->prec = value((double)(o->prec) * 0.3, find_cal_pol(s,  r->datime, "r"));
  if (r->prec >= 0.0 && r->prec < 200.0)
    r->mask |= MASK_RAIN;

  if (o->mask == MASK_WWSR_FAKERAIN)
    r->mask |= MASK_FAKERAIN;

  return 1;
}

/*!
  \fn int interp_data(struct real_data *r, struct obs_data *o, size_t nobs, time_t datime, struct station_cal *s, double altitude)
  \brief Get a interpolated struct \ref real_data for a given time
  \param r pointer to the struct \ref real_data where to set the result
  \param o pointer to an array of struct \ref obs_data readed from a DB file
  \param nobs elements in the array of \ref obs_data pointed by \a o
  \param datime Unix time for target
  \param s pointer to a struct \ref station_cal with the calibration data
  \param altitude altitude in meters of the wwsr panel station

  Return 1 if succeded , 0 otherwise
*/
int interp_data(struct real_data *r, struct obs_data *o, size_t nobs, time_t datime,
                struct station_cal *s, double altitude)
{
  double tx;
  size_t i = 0;
  struct real_data r1, r2;

  while (o[i].datime < datime && i < nobs) i++;

  if (i < nobs && o[i].datime == datime)
    {
      obsdata_to_realdata( r, &o[i], s, altitude);
      return 1;
    }

  if (i < nobs && i > 0)
    {
      memset(&r1, 0, sizeof(struct real_data));
      memset(&r2, 0, sizeof(struct real_data));
      obsdata_to_realdata( &r1, &o[i - 1], s, altitude);
      obsdata_to_realdata( &r2, &o[i], s, altitude);

      r->mask = (r1.mask & r2.mask);
      if ((r->mask & MASK_DATIME) == 0)
        return 0;
      tx = ((double)datime - r1.datime) / ((double)r2.datime - r1.datime);
      r->datime = datime;
      r->interval = datime - r1.datime;

      if (r->mask & MASK_HUM_INT)
        r->hum_int = r1.hum_int + (r2.hum_int - r1.hum_int) * tx;

      if (r->mask & MASK_HUM_EXT)
        r->hum_out = r1.hum_out + (r2.hum_out - r1.hum_out) * tx;

      if (r->mask & MASK_TEMP_INT)
        r->temp_int = r1.temp_int + (r2.temp_int - r1.temp_int) * tx;

      if (r->mask & MASK_TEMP_EXT)
        r->temp_out = r1.temp_out + (r2.temp_out - r1.temp_out) * tx;

      if ((r->mask & MASK_HUM_INT) && (r->mask & MASK_TEMP_INT))
        r->tdew_int = r1.tdew_int + (r2.tdew_int - r1.tdew_int) * tx;

      if ((r->mask & MASK_HUM_EXT) && (r->mask & MASK_TEMP_EXT))
        r->tdew_out = r1.tdew_out + (r2.tdew_out - r1.tdew_out) * tx;

      if (r->mask & MASK_PRES)
        {
          r->pres0 = r1.pres0 + (r2.pres0 - r1.pres0) * tx;
          r->press = r1.press + (r2.press - r1.press) * tx;
        }
      if (r->mask & MASK_WSPD)
        {
          r->wspd = r1.wspd + (r2.wspd - r1.wspd) * tx;
          r->wgust = r1.wgust + (r2.wgust - r1.wgust) * tx;
        }

      if (r->mask & MASK_WDIR)
        r->wdir = r1.wdir + (r2.wdir - r1.wdir) * tx;

      if (r->mask & MASK_RAIN)
        r->prec = r1.prec + (r2.prec - r1.prec) * tx;
    }
  else
    return 0;
  return 1;
}

int read_station_conf(struct station_conf *e, const char *filename)
{
  FILE *f;
  int nitem = 0;
  double x;
  char lin[256], name[256], val[256], *c , *c1;

  if ((f = fopen(filename, "r")) == NULL)
    return nitem; // cannot open the file

  /* clean */
  memset((void *) e, 0, sizeof(struct station_conf));

  strncat(e->confile, filename, PATH_LEN - 1);
  nitem++;

  while ( fgets(lin, 256, f) != NULL)
    {
      if (lin[0] == '#')
        continue; // is a comment
      c = strchr(lin,'=');
      c1 = strchr(lin,'\n');
      if (c != NULL)
        {
          *c = '\0';
          *c1 = '\0';
          strcpy(name, lin);

          // strip blanks and "
          c += 1;
          while ((*c == ' ') || (*c == '\"')) c += 1;
          c1 -= 1;
          while ((*c1 == ' ') || (*c1 == '\"')) *(c1--) = '\0';

          strcpy(val ,c);

          /*printf(" \"%s\" , \"%s\"\n", name, val);*/
          if (strcmp(name,"DBFILE") == 0)
            {
              strncat(e->dbfile, val, PATH_LEN - 1);
              nitem++;
            }
          else if (strcmp(name, "CALFILE") == 0)
            {
              strncat(e->calfile, val, PATH_LEN - 1);
              nitem++;
            }
          else if (strcmp(name, "ESTACION") == 0 ||
                   strcmp(name, "INDEX") == 0)
            {
              strncat(e->index, val, PATH_LEN - 1);
              nitem++;
            }
          else if (strcmp(name, "NOMBRE") == 0 ||
                   strcmp(name, "NAME") == 0)
            {
              strncat(e->name, val, PATH_LEN - 1);
              nitem++;
            }
          else if (strcmp(name, "ALTITUD") == 0 ||
                   strcmp(name, "ALTITUDE") == 0)
            {
              x = strtod(val, &c);
              if (errno == 0)
                {
                  e->altitude = x;
                  nitem++;
                }
            }
          else if (strcmp(name, "LATITUD") == 0 ||
                   strcmp(name, "LATITUDE") == 0)
            {
              x = strtod(val, &c);
              if (errno == 0)
                {
                  e->latitude = x;
                  nitem++;
                }
            }
          else if (strcmp(name, "LONGITUD") == 0 ||
                   strcmp(name, "LONGITUDE") == 0)
            {
              x = strtod(val, &c);
              if (errno == 0)
                {
                  e->longitude = x;
                  nitem++;
                }
            }
        }
    }
  fclose(f);
  return nitem;
}
